@@ -19,27 +19,31 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
 import asyncio
 import contextlib
 import os
 import signal
 import time
-from typing import TYPE_CHECKING, Callable, Dict, Iterator, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING
 
 import xcffib
 import xcffib.render
 import xcffib.xproto
-from xcffib.xproto import EventMask, StackMode
+import xcffib.xtest
+from xcffib.xproto import EventMask
 
 from libqtile import config, hook, utils
 from libqtile.backend import base
 from libqtile.backend.x11 import window, xcbq
 from libqtile.backend.x11.xkeysyms import keysyms
+from libqtile.config import ScreenRect
 from libqtile.log_utils import logger
 from libqtile.utils import QtileError
 
 if TYPE_CHECKING:
-    from libqtile.core.manager import Qtile
+    from collections.abc import Callable, Iterator
 
 _IGNORED_EVENTS = {
     xcffib.xproto.CreateNotifyEvent,
@@ -52,11 +56,11 @@ _IGNORED_EVENTS = {
 }
 
 
-def get_keys() -> List[str]:
+def get_keys() -> list[str]:
     return list(xcbq.keysyms.keys())
 
 
-def get_modifiers() -> List[str]:
+def get_modifiers() -> list[str]:
     return list(xcbq.ModMasks.keys())
 
 
@@ -65,7 +69,7 @@ class ExistingWMException(Exception):
 
 
 class Core(base.Core):
-    def __init__(self, display_name: str = None) -> None:
+    def __init__(self, display_name: str | None = None) -> None:
         """Setup the X11 core backend
 
         :param display_name:
@@ -96,7 +100,7 @@ class Core(base.Core):
                 "_NET_WM_NAME", "UTF8_STRING", unpack=str
             )
             if existing_wmname:
-                logger.error("not starting; existing window manager {}".format(existing_wmname))
+                logger.error("not starting; existing window manager %s", existing_wmname)
                 raise ExistingWMException(existing_wmname)
 
         self.eventmask = (
@@ -116,6 +120,7 @@ class Core(base.Core):
         self._wmname = "qtile"
         self._supporting_wm_check_window = self.conn.create_window(-1, -1, 1, 1)
         self._supporting_wm_check_window.set_property("_NET_WM_NAME", self._wmname)
+        self._supporting_wm_check_window.set_property("_NET_WM_PID", os.getpid())
         self._supporting_wm_check_window.set_property(
             "_NET_SUPPORTING_WM_CHECK", self._supporting_wm_check_window.wid
         )
@@ -128,8 +133,8 @@ class Core(base.Core):
         self._selection_window = self.conn.create_window(-1, -1, 1, 1)
         self._selection_window.set_attribute(eventmask=EventMask.PropertyChange)
         if hasattr(self.conn, "xfixes"):
-            self.conn.xfixes.select_selection_input(self._selection_window, "PRIMARY")  # type: ignore
-            self.conn.xfixes.select_selection_input(self._selection_window, "CLIPBOARD")  # type: ignore
+            self.conn.xfixes.select_selection_input(self._selection_window, "PRIMARY")
+            self.conn.xfixes.select_selection_input(self._selection_window, "CLIPBOARD")
 
         primary_atom = self.conn.atoms["PRIMARY"]
         reply = self.conn.conn.core.GetSelectionOwner(primary_atom).reply()
@@ -146,42 +151,44 @@ class Core(base.Core):
         # setup the default cursor
         self._root.set_cursor("left_ptr")
 
-        self.qtile = None  # type: Optional[Qtile]
         self._painter = None
+        self._xtest = self.conn.conn(xcffib.xtest.key)
 
-        numlock_code = self.conn.keysym_to_keycode(xcbq.keysyms["Num_Lock"])[0]
+        numlock_code = self.conn.keysym_to_keycode(xcbq.keysyms["num_lock"])[0]
         self._numlock_mask = xcbq.ModMasks.get(self.conn.get_modifier(numlock_code), 0)
-        self._valid_mask = ~(self._numlock_mask | xcbq.ModMasks["lock"] | xcbq.AllButtonsMask)
+        self._valid_mask = ~(
+            self._numlock_mask
+            | xcbq.ModMasks["lock"]
+            | xcbq.AllButtonsMask
+            | xcbq.PointerMotionHintMask
+        )
+
+        # The last motion notify event that we still need to handle
+        self._motion_notify: xcffib.Event | None = None
+        # The last time we were handling a MotionNotify event
+        self._last_motion_time = 0
+
+        self.last_focused: base.Window | None = None
 
     @property
     def name(self):
         return "x11"
 
     def finalize(self) -> None:
-        self.conn.conn.core.DeletePropertyChecked(
-            self._root.wid,
-            self.conn.atoms["_NET_SUPPORTING_WM_CHECK"],
-        ).check()
-        self.qtile = None
+        with contextlib.suppress(xcffib.ConnectionException):
+            self.conn.conn.core.DeletePropertyChecked(
+                self._root.wid,
+                self.conn.atoms["_NET_SUPPORTING_WM_CHECK"],
+            ).check()
+        if hasattr(self, "qtile"):
+            delattr(self, "qtile")
         self.conn.finalize()
 
-    def get_screen_info(self) -> List[Tuple[int, int, int, int]]:
-        info = [(s.x, s.y, s.width, s.height) for s in self.conn.pseudoscreens]
-
-        if not info:
-            info.append(
-                (
-                    0,
-                    0,
-                    self.conn.default_screen.width_in_pixels,
-                    self.conn.default_screen.height_in_pixels,
-                )
-            )
-
+    def get_screen_info(self) -> list[ScreenRect]:
+        ps = self.conn.pseudoscreens
         if self.qtile:
             self._xpoll()
-
-        return info
+        return ps
 
     @property
     def wmname(self):
@@ -192,7 +199,7 @@ class Core(base.Core):
         self._wmname = wmname
         self._supporting_wm_check_window.set_property("_NET_WM_NAME", wmname)
 
-    def setup_listener(self, qtile: "Qtile") -> None:
+    def setup_listener(self) -> None:
         """Setup a listener for the given qtile instance
 
         :param qtile:
@@ -201,7 +208,6 @@ class Core(base.Core):
             The eventloop to use to listen to the file descriptor.
         """
         logger.debug("Adding io watch")
-        self.qtile = qtile
         self.fd = self.conn.conn.get_file_descriptor()
         asyncio.get_running_loop().add_reader(self.fd, self._xpoll)
 
@@ -213,20 +219,28 @@ class Core(base.Core):
             loop.remove_reader(self.fd)
             self.fd = None
 
-    def distribute_windows(self, initial) -> None:
+    def on_config_load(self, initial) -> None:
         """Assign windows to groups"""
         assert self.qtile is not None
 
+        # Ensure that properties are initialised at startup
+        self.update_client_lists()
+
         if not initial:
             # We are just reloading config
-            for win in self.qtile.windows_map.values():
-                if type(win) is window.Window:
-                    win.set_group()
+            managed_wins = [
+                w for w in self.qtile.windows_map.values() if isinstance(w, window.Window)
+            ]
+            for managed_win in managed_wins:
+                if managed_win.group and managed_win in managed_win.group.windows:
+                    # Remove window from old group
+                    managed_win.group.remove(managed_win)
+                managed_win.set_group()
             return
 
         # Qtile just started - scan for clients
-        _, _, children = self._root.query_tree()
-        for item in children:
+        for wid in self._root.query_tree():
+            item = window.XWindow(self.conn, wid)
             try:
                 attrs = item.get_attributes()
                 state = item.get_wm_state()
@@ -256,11 +270,14 @@ class Core(base.Core):
 
                 if item.get_wm_type() == "dock" or win.reserved_space:
                     assert self.qtile.current_screen is not None
-                    win.cmd_static(self.qtile.current_screen.index)
+                    win.static(self.qtile.current_screen.index)
                     continue
 
             self.qtile.manage(win)
 
+            self.update_client_lists()
+            win.change_layer()
+
     def warp_pointer(self, x, y):
         self._root.warp_pointer(x, y)
         self._root.set_input_focus()
@@ -276,6 +293,18 @@ class Core(base.Core):
             xcffib.CurrentTime,
         )
 
+    def handle_event(self, event):
+        """Handle an X11 event by forwarding it to the right target"""
+        event_type = event.__class__.__name__
+        if event_type.endswith("Event"):
+            event_type = event_type[:-5]
+        targets = self._get_target_chain(event_type, event)
+        logger.debug("X11 event: %s (targets: %s)", event_type, targets)
+        for target in targets:
+            ret = target(event)
+            if not ret:
+                break
+
     def _xpoll(self) -> None:
         """Poll the connection and dispatch incoming events"""
         assert self.qtile is not None
@@ -289,16 +318,23 @@ class Core(base.Core):
                 if event.__class__ in _IGNORED_EVENTS:
                     continue
 
-                event_type = event.__class__.__name__
-                if event_type.endswith("Event"):
-                    event_type = event_type[:-5]
-
-                targets = self._get_target_chain(event_type, event)
-                logger.debug(f"X11 event: {event_type} (targets: {len(targets)})")
-                for target in targets:
-                    ret = target(event)
-                    if not ret:
-                        break
+                # Motion Notifies are handled later
+                # Otherwise this is too CPU intensive
+                if isinstance(event, xcffib.xproto.MotionNotifyEvent):
+                    self._motion_notify = event
+                else:
+                    # These events need the motion notify event handled first
+                    handle_motion_first = type(event) in [
+                        xcffib.xproto.EnterNotifyEvent,
+                        xcffib.xproto.LeaveNotifyEvent,
+                        xcffib.xproto.ButtonPressEvent,
+                        xcffib.xproto.ButtonReleaseEvent,
+                    ]
+                    # Handle events in the correct order
+                    if self._motion_notify and handle_motion_first:
+                        self.handle_event(self._motion_notify)
+                        self._motion_notify = None
+                    self.handle_event(event)
 
             # Catch some bad X exceptions. Since X is event based, race
             # conditions can occur almost anywhere in the code. For example, if
@@ -319,19 +355,18 @@ class Core(base.Core):
             except Exception:
                 error_code = self.conn.conn.has_error()
                 if error_code:
-                    error_string = xcbq.XCB_CONN_ERRORS[error_code]
-                    logger.exception(
-                        "Shutting down due to X connection error {error_string} ({error_code})".format(
-                            error_string=error_string, error_code=error_code
-                        )
-                    )
+                    logger.warning("Shutting down due to disconnection from X server")
                     self.remove_listener()
                     self.qtile.stop()
                     return
                 logger.exception("Got an exception in poll loop")
+        # Handle any outstanding motion notify events
+        if self._motion_notify:
+            self.handle_event(self._motion_notify)
+            self._motion_notify = None
         self.flush()
 
-    def _get_target_chain(self, event_type: str, event) -> List[Callable]:
+    def _get_target_chain(self, event_type: str, event) -> list[Callable]:
         """Returns a chain of targets that can handle this event
 
         Finds functions named `handle_X`, either on the window object itself or
@@ -344,7 +379,7 @@ class Core(base.Core):
         """
         assert self.qtile is not None
 
-        handler = "handle_{event_type}".format(event_type=event_type)
+        handler = f"handle_{event_type}"
         # Certain events expose the affected window id as an "event" attribute.
         event_events = [
             "EnterNotify",
@@ -408,17 +443,28 @@ class Core(base.Core):
         """The name of the connected display"""
         return self._display_name
 
-    def update_client_list(self, windows_map: Dict[int, base.WindowType]) -> None:
-        """Updates the client stack list
+    def update_client_lists(self) -> None:
+        """Updates the _NET_CLIENT_LIST and _NET_CLIENT_LIST_STACKING properties
 
         This is needed for third party tasklists and drag and drop of tabs in
         chrome
         """
+        assert self.qtile
         # Regular top-level managed windows, i.e. excluding Static, Internal and Systray Icons
-        wids = [wid for wid, c in windows_map.items() if isinstance(c, window.Window)]
+        wids = [wid for wid, c in self.qtile.windows_map.items() if isinstance(c, window.Window)]
         self._root.set_property("_NET_CLIENT_LIST", wids)
-        # TODO: check stack order
-        self._root.set_property("_NET_CLIENT_LIST_STACKING", wids)
+
+        # We rely on the stacking order from the X server
+        stacked_wids = []
+        for wid in self._root.query_tree():
+            win = self.qtile.windows_map.get(wid)
+            if not win:
+                continue
+
+            if isinstance(win, window.Window) and win.group:
+                stacked_wids.append(wid)
+
+        self._root.set_property("_NET_CLIENT_LIST_STACKING", stacked_wids)
 
     def update_desktops(self, groups, index: int) -> None:
         """Set the current desktops of the window manager
@@ -434,24 +480,30 @@ class Core(base.Core):
             viewport += [group.screen.x, group.screen.y] if group.screen else [0, 0]
         self._root.set_property("_NET_DESKTOP_VIEWPORT", viewport)
 
-    def lookup_key(self, key: Union[config.Key, config.KeyChord]) -> Tuple[int, int]:
+    def lookup_key(self, key: config.Key | config.KeyChord) -> tuple[int, int]:
         """Find the keysym and the modifier mask for the given key"""
-        try:
-            keysym = xcbq.get_keysym(key.key)
-            modmask = xcbq.translate_masks(key.modifiers)
-        except xcbq.XCBQError as err:
-            raise utils.QtileError(err)
+        if isinstance(key.key, str):
+            keysym = xcbq.keysyms.get(key.key.lower())
+            if not keysym:
+                raise utils.QtileError(f"Unknown keysym: {key.key}")
+
+        else:
+            keysym = self.conn.code_to_syms[key.key][0]
+            if not keysym:
+                raise utils.QtileError(f"Unknown keycode: {key.key}")
+
+        modmask = xcbq.translate_masks(key.modifiers)
 
         return keysym, modmask
 
-    def grab_key(self, key: Union[config.Key, config.KeyChord]) -> Tuple[int, int]:
+    def grab_key(self, key: config.Key | config.KeyChord) -> tuple[int, int]:
         """Map the key to receive events on it"""
         keysym, modmask = self.lookup_key(key)
         codes = self.conn.keysym_to_keycode(keysym)
 
         for code in codes:
             if code == 0:
-                logger.warning(f"Can't grab {key} (unknown keysym: {hex(keysym)})")
+                logger.warning("Can't grab %s (unknown keysym: %02x)", key, keysym)
                 continue
             for amask in self._auto_modmasks():
                 self.conn.conn.core.GrabKey(
@@ -464,7 +516,7 @@ class Core(base.Core):
                 )
         return keysym, modmask & self._valid_mask
 
-    def ungrab_key(self, key: Union[config.Key, config.KeyChord]) -> Tuple[int, int]:
+    def ungrab_key(self, key: config.Key | config.KeyChord) -> tuple[int, int]:
         """Ungrab the key corresponding to the given keysym and modifier mask"""
         keysym, modmask = self.lookup_key(key)
         codes = self.conn.keysym_to_keycode(keysym)
@@ -544,7 +596,7 @@ class Core(base.Core):
             i._reset_mask()
 
     def create_internal(
-        self, x: int, y: int, width: int, height: int, desired_depth: Optional[int] = 32
+        self, x: int, y: int, width: int, height: int, desired_depth: int | None = 32
     ) -> base.Internal:
         assert self.qtile is not None
 
@@ -596,15 +648,30 @@ class Core(base.Core):
         if atoms["_NET_CURRENT_DESKTOP"] == opcode:
             index = data.data32[0]
             try:
-                self.qtile.groups[index].cmd_toscreen()
+                self.qtile.groups[index].toscreen()
             except IndexError:
-                logger.debug("Invalid desktop index: %s" % index)
+                logger.debug("Invalid desktop index: %s", index)
 
-    def handle_KeyPress(self, event) -> None:  # noqa: N802
+    def handle_KeyPress(self, event, *, simulated=False) -> None:  # noqa: N802
         assert self.qtile is not None
-
         keysym = self.conn.code_to_syms[event.detail][0]
-        self.qtile.process_key_event(keysym, event.state & self._valid_mask)
+        key, handled = self.qtile.process_key_event(keysym, event.state & self._valid_mask)
+
+        if simulated:
+            # Even though simulated keybindings could use a proper X11 event, we don't want do any fake input
+            # This is because it needs extra code for e.g. pressing/releasing the modifiers
+            # This we don't want to handle and instead leave to external tools such as xdotool
+            return
+
+        # As we're grabbing async we can't just replay it, so...
+        # We need to forward the event to the focused window
+        if not handled and key:
+            # We need to ungrab the key as otherwise we get an event loop
+            self.ungrab_key(key)
+            # Modifier is pressed, just repeat the event with xtest
+            self._fake_KeyPress(event)
+            # Grab the key again
+            self.grab_key(key)
 
     def handle_ButtonPress(self, event) -> None:  # noqa: N802
         assert self.qtile is not None
@@ -628,6 +695,15 @@ class Core(base.Core):
     def handle_MotionNotify(self, event) -> None:  # noqa: N802
         assert self.qtile is not None
 
+        # Limit the motion notify events from happening too frequently
+        # As we already handle motion notify events "later", the default is None
+        # So we also need to check if this has to be done in the first place
+        resize_fps = self.qtile.current_screen.x11_drag_polling_rate
+        if resize_fps is not None and (event.time - self._last_motion_time) <= (
+            1000 / resize_fps
+        ):
+            return
+        self._last_motion_time = event.time
         self.qtile.process_button_motion(event.event_x, event.event_y)
 
     def handle_ConfigureRequest(self, event):  # noqa: N802
@@ -682,18 +758,21 @@ class Core(base.Core):
 
             if xwin.get_wm_type() == "dock" or win.reserved_space:
                 assert self.qtile.current_screen is not None
-                win.cmd_static(self.qtile.current_screen.index)
+                win.static(self.qtile.current_screen.index)
                 return
 
             self.qtile.manage(win)
             if not win.group or not win.group.screen:
                 return
             win.unhide()
+            self.update_client_lists()
+            win.change_layer()
 
     def handle_DestroyNotify(self, event) -> None:  # noqa: N802
         assert self.qtile is not None
 
         self.qtile.unmanage(event.window)
+        self.update_client_lists()
         if self.qtile.current_window is None:
             self.conn.fixup_focus()
 
@@ -713,19 +792,41 @@ class Core(base.Core):
                 # since the window is dead.
                 pass
             # Clear these atoms as per spec
-            win.window.conn.conn.core.DeleteProperty(  # type: ignore
-                win.wid, win.window.conn.atoms["_NET_WM_STATE"]  # type: ignore
+            win.window.conn.conn.core.DeleteProperty(
+                win.wid, win.window.conn.atoms["_NET_WM_STATE"]
             )
-            win.window.conn.conn.core.DeleteProperty(  # type: ignore
-                win.wid, win.window.conn.atoms["_NET_WM_DESKTOP"]  # type: ignore
+            win.window.conn.conn.core.DeleteProperty(
+                win.wid, win.window.conn.atoms["_NET_WM_DESKTOP"]
             )
         self.qtile.unmanage(event.window)
+        self.update_client_lists()
         if self.qtile.current_window is None:
             self.conn.fixup_focus()
 
     def handle_ScreenChangeNotify(self, event) -> None:  # noqa: N802
         hook.fire("screen_change", event)
 
+    def _fake_input(self, input_type, detail, x=0, y=0) -> None:
+        self._xtest.FakeInput(
+            input_type,
+            detail,
+            0,  # This is a delay, not timestamp, according to AwesomeWM
+            xcffib.XCB_NONE,
+            x,  # x: Only used for motion events
+            y,  # y: Only used for motion events
+            0,
+        )
+        self.flush()
+
+    def _fake_KeyPress(self, event) -> None:  # noqa: N802
+        # First release the key as it is possibly already pressed
+        for input_type in (
+            xcbq.XCB_KEY_RELEASE,
+            xcbq.XCB_KEY_PRESS,
+            xcbq.XCB_KEY_RELEASE,
+        ):
+            self._fake_input(input_type, event.detail)
+
     @contextlib.contextmanager
     def disable_unmap_events(self):
         self._root.set_attribute(eventmask=self.eventmask & (~EventMask.SubstructureNotify))
@@ -740,9 +841,8 @@ class Core(base.Core):
 
     def simulate_keypress(self, modifiers, key):
         """Simulates a keypress on the focused window."""
-        # FIXME: This needs to be done with sendevent, once we have that fixed.
         modmasks = xcbq.translate_masks(modifiers)
-        keysym = xcbq.keysyms.get(key)
+        keysym = xcbq.keysyms.get(key.lower())
 
         class DummyEv:
             pass
@@ -750,7 +850,7 @@ class Core(base.Core):
         d = DummyEv()
         d.detail = self.conn.keysym_to_keycode(keysym)[0]
         d.state = modmasks
-        self.handle_KeyPress(d)
+        self.handle_KeyPress(d, simulated=True)
 
     def focus_by_click(self, e, window=None):
         """Bring a window to the front
@@ -768,9 +868,7 @@ class Core(base.Core):
                 qtile.config.bring_front_click != "floating_only"
                 or getattr(window, "floating", False)
             ):
-                self.conn.conn.core.ConfigureWindow(
-                    window.wid, xcffib.xproto.ConfigWindow.StackMode, [StackMode.Above]
-                )
+                window.bring_to_front()
 
             try:
                 if window.group.screen is not qtile.current_screen:
@@ -787,6 +885,8 @@ class Core(base.Core):
             # clicked on root window
             screen = qtile.find_screen(e.root_x, e.root_y)
             if screen:
+                if qtile.current_window:
+                    qtile.current_window._grab_click()
                 qtile.focus_screen(screen.index, warp=False)
 
     def flush(self):
@@ -795,19 +895,15 @@ class Core(base.Core):
     def graceful_shutdown(self):
         """Try to close windows gracefully before exiting"""
 
-        def get_interesting_pid(win):
-            # We don't need to kill Internal or Static windows, they're qtile
-            # managed and don't have any state.
-            if not isinstance(win, base.Window):
-                return None
-            try:
-                return win.window.get_net_wm_pid()
-            except Exception:
-                logger.exception("Got an exception in getting the window pid")
-                return None
-
-        pids = map(get_interesting_pid, self.qtile.windows_map.values())
-        pids = list(filter(lambda x: x is not None, pids))
+        try:
+            pids = []
+            for win in self.qtile.windows_map.values():
+                if not isinstance(win, base.Internal):
+                    if pid := win.get_pid():
+                        pids.append(pid)
+        except xcffib.ConnectionException:
+            logger.warning("Server disconnected, couldn't close windows gracefully.")
+            return
 
         # Give the windows a chance to shut down nicely.
         for pid in pids:
@@ -833,7 +929,7 @@ class Core(base.Core):
                 break
             time.sleep(0.1)
 
-    def get_mouse_position(self) -> Tuple[int, int]:
+    def get_mouse_position(self) -> tuple[int, int]:
         """
         Get mouse coordinates.
         """
@@ -842,4 +938,19 @@ class Core(base.Core):
 
     def keysym_from_name(self, name: str) -> int:
         """Get the keysym for a key from its name"""
-        return keysyms[name]
+        return keysyms[name.lower()]
+
+    def check_stacking(self, win: base.Window) -> None:
+        """Triggers restacking if a fullscreen window loses focus."""
+        if win is self.last_focused:
+            return
+
+        if self.last_focused and self.last_focused.fullscreen:
+            self.last_focused.change_layer()
+
+        self.last_focused = win
+
+    @property
+    def hovered_window(self) -> base.WindowType | None:
+        _hovered_window = self.conn.conn.core.QueryPointer(self._root.wid).reply().child
+        return self.qtile.windows_map.get(_hovered_window)
