@@ -31,12 +31,15 @@
 # SOFTWARE.
 
 """
-    A minimal EWMH-aware OO layer over xcffib. This is NOT intended to be
-    complete - it only implements the subset of functionalty needed by qtile.
+A minimal EWMH-aware OO layer over xcffib. This is NOT intended to be
+complete - it only implements the subset of functionalty needed by qtile.
 """
+
+from __future__ import annotations
+
+import contextlib
 import functools
 import operator
-import typing
 from itertools import chain, repeat
 
 import cairocffi
@@ -52,23 +55,15 @@ from xcffib.xproto import CW, EventMask, WindowClass
 from libqtile.backend.x11 import window
 from libqtile.backend.x11.xcursors import Cursors
 from libqtile.backend.x11.xkeysyms import keysyms
+from libqtile.config import ScreenRect
 from libqtile.log_utils import logger
-from libqtile.utils import QtileError, hex
+from libqtile.utils import QtileError, hex, rgb
 
 
 class XCBQError(QtileError):
     pass
 
 
-def rdict(d):
-    r = {}
-    for k, v in d.items():
-        r.setdefault(v, []).append(k)
-    return r
-
-
-rkeysyms = rdict(keysyms)
-
 # Keyboard modifiers bitmask values from X Protocol
 ModMasks = {
     "shift": 1 << 0,
@@ -84,6 +79,7 @@ ModMasks = {
 AllButtonsMask = 0b11111 << 8
 ButtonMotionMask = 1 << 13
 ButtonReleaseMask = 1 << 3
+PointerMotionHintMask = 1 << 7
 
 NormalHintsFlags = {
     "USPosition": 1,  # User-specified x, y
@@ -219,6 +215,13 @@ XCB_CONN_ERRORS = {
     7: "XCB_CONN_CLOSED_FDPASSING_FAILED",
 }
 
+# Some opcodes from xproto.h, used for faking input.
+XCB_KEY_PRESS = 2
+XCB_KEY_RELEASE = 3
+XCB_BUTTON_PRESS = 4
+XCB_BUTTON_RELEASE = 5
+XCB_MOTION_NOTIFY = 6
+
 
 class MaskMap:
     """
@@ -251,7 +254,7 @@ class MaskMap:
                     values.append(getattr(val, "_maskvalue", val))
                 del kwargs[s]
         if kwargs:
-            raise ValueError("Unknown mask names: %s" % list(kwargs.keys()))
+            raise ValueError(f"Unknown mask names: {list(kwargs.keys())}")
         return mask, values
 
 
@@ -334,8 +337,9 @@ class Screen(_Wrapper):
             return desired_depth, self._visuals[desired_depth]
 
         logger.info(
-            f"{desired_depth} bit colour depth not available. "
-            f"Falling back to root depth: {self.root_depth}."
+            "%s bit colour depth not available. Falling back to root depth: %s.",
+            desired_depth,
+            self.root_depth,
         )
         return self.root_depth, self._visuals[self.root_depth]
 
@@ -349,7 +353,7 @@ class Screen(_Wrapper):
         """
         allowed = screen.allowed_depths
         if depth not in [x.depth for x in allowed]:
-            logger.warning(f"Unsupported colour depth: {depth}.")
+            logger.warning("Unsupported colour depth: %s", depth)
             return
 
         for i in allowed:
@@ -358,20 +362,6 @@ class Screen(_Wrapper):
                     return i.visuals[0]
 
 
-class PseudoScreen:
-    """
-    This may be a Xinerama screen or a RandR CRTC, both of which are
-    rectangular sections of an actual Screen.
-    """
-
-    def __init__(self, conn, x, y, width, height):
-        self.conn = conn
-        self.x = x
-        self.y = y
-        self.width = width
-        self.height = height
-
-
 class Colormap:
     def __init__(self, conn, cid):
         self.conn = conn
@@ -388,7 +378,12 @@ class Colormap:
             def x8to16(i):
                 return 0xFFFF * (i & 0xFF) // 0xFF
 
-            color = hex(color)
+            try:
+                color = hex(color)
+            except ValueError:
+                logger.error("Colormap failed to allocate %s", color)
+                color = "#ff0000"
+
             r = x8to16(int(color[-6] + color[-5], 16))
             g = x8to16(int(color[-4] + color[-3], 16))
             b = x8to16(int(color[-2] + color[-1], 16))
@@ -410,17 +405,12 @@ class RandR:
         self.ext.SelectInput(conn.default_screen.root.wid, xcffib.randr.NotifyMask.ScreenChange)
 
     def query_crtcs(self, root):
-        crtc_list = []
+        infos = []
         for crtc in self.ext.GetScreenResources(root).reply().crtcs:
             crtc_info = self.ext.GetCrtcInfo(crtc, xcffib.CurrentTime).reply()
-            crtc_dict = {
-                "x": crtc_info.x,
-                "y": crtc_info.y,
-                "width": crtc_info.width,
-                "height": crtc_info.height,
-            }
-            crtc_list.append(crtc_dict)
-        return crtc_list
+
+            infos.append(ScreenRect(crtc_info.x, crtc_info.y, crtc_info.width, crtc_info.height))
+        return infos
 
 
 class XFixes:
@@ -440,42 +430,6 @@ class XFixes:
         self.conn.xfixes.ext.SelectSelectionInput(window.wid, _selection, self.selection_mask)
 
 
-class NetWmState:
-    """NetWmState is a descriptor for _NET_WM_STATE_* properties"""
-
-    def __init__(self, prop_name):
-        self.prop_name = prop_name
-
-    def __get__(self, xcbq_win, cls):
-        try:
-            atom = self.atom
-        except AttributeError:
-            atom = xcbq_win.conn.atoms[self.prop_name]
-            self.atom = atom
-        reply = xcbq_win.get_property("_NET_WM_STATE", "ATOM", unpack=int)
-        if atom in reply:
-            return True
-        return False
-
-    def __set__(self, xcbq_win, value):
-        try:
-            atom = self.atom
-        except AttributeError:
-            atom = xcbq_win.conn.atoms[self.prop_name]
-            self.atom = atom
-
-        value = bool(value)
-        reply = list(xcbq_win.get_property("_NET_WM_STATE", "ATOM", unpack=int))
-        is_set = atom in reply
-        if is_set and not value:
-            reply.remove(atom)
-            xcbq_win.set_property("_NET_WM_STATE", reply)
-        elif value and not is_set:
-            reply.append(atom)
-            xcbq_win.set_property("_NET_WM_STATE", reply)
-        return
-
-
 class Connection:
     _extmap = {
         "xinerama": Xinerama,
@@ -527,28 +481,19 @@ class Connection:
 
     @property
     def pseudoscreens(self):
-        pseudoscreens = []
         if hasattr(self, "xinerama"):
+            pseudoscreens = []
             for i, s in enumerate(self.xinerama.query_screens()):
-                scr = PseudoScreen(
-                    self,
+                scr = ScreenRect(
                     s.x_org,
                     s.y_org,
                     s.width,
                     s.height,
                 )
                 pseudoscreens.append(scr)
+            return pseudoscreens
         elif hasattr(self, "randr"):
-            for i in self.randr.query_crtcs(self.screens[0].root.wid):
-                scr = PseudoScreen(
-                    self,
-                    i["x"],
-                    i["y"],
-                    i["width"],
-                    i["height"],
-                )
-                pseudoscreens.append(scr)
-        return pseudoscreens
+            return self.randr.query_crtcs(self.screens[0].root.wid)
 
     def finalize(self):
         self.cursors.finalize()
@@ -632,10 +577,8 @@ class Connection:
         return window.XWindow(self, wid)
 
     def disconnect(self):
-        try:
+        with contextlib.suppress(xcffib.ConnectionException):
             self.conn.disconnect()
-        except xcffib.ConnectionException:
-            logger.error("Failed to disconnect, connection already failed?")
         self._connected = False
 
     def flush(self):
@@ -670,7 +613,7 @@ class Connection:
                 xcffib.xproto.Time.CurrentTime,
             )
 
-    @functools.lru_cache()
+    @functools.lru_cache
     def color_pixel(self, name):
         pixel = self.screens[0].default_colormap.alloc_color(name).pixel
         return pixel | 0xFF << 24
@@ -684,32 +627,42 @@ class Painter:
         self.default_screen = self.screens[self.conn.pref_screen]
         self.conn.core.SetCloseDownMode(xcffib.xproto.CloseDown.RetainPermanent)
         self.atoms = AtomCache(self)
+        self.width = -1
+        self.height = -1
+        self.root_pixmap_id = None
+
+    def _get_root_pixmap_and_surface(self, screen):
+        # Querying the screen dimensions via the xcffib connection does not
+        # take account of any screen scaling. We can therefore work out the
+        # necessary size of the root window by looking at the
+        # pseudoscreens attribute and calculating the max x and y extents.
+        root_windows = screen.qtile.core.conn.pseudoscreens
+        width = max(win.x + win.width for win in root_windows)
+        height = max(win.y + win.height for win in root_windows)
 
-    def paint(self, screen, image_path, mode=None):
         try:
-            with open(image_path, "rb") as f:
-                image, _ = cairocffi.pixbuf.decode_to_image_surface(f.read())
-        except IOError as e:
-            logger.error("Wallpaper: %s" % e)
-            return
+            root_pixmap = self.default_screen.root.get_property(
+                "_XROOTPMAP_ID", xcffib.xproto.Atom.PIXMAP, int
+            )
+        except xcffib.ConnectionException:
+            root_pixmap = None
 
-        root_pixmap = self.default_screen.root.get_property(
-            "_XROOTPMAP_ID", xcffib.xproto.Atom.PIXMAP, int
-        )
         if not root_pixmap:
             root_pixmap = self.default_screen.root.get_property(
                 "ESETROOT_PMAP_ID", xcffib.xproto.Atom.PIXMAP, int
             )
-        if root_pixmap:
+        if root_pixmap and (self.width == width and self.height == height):
             root_pixmap = root_pixmap[0]
         else:
+            self.width = width
+            self.height = height
             root_pixmap = self.conn.generate_id()
             self.conn.core.CreatePixmap(
                 self.default_screen.root_depth,
                 root_pixmap,
                 self.default_screen.root.wid,
-                self.default_screen.width_in_pixels,
-                self.default_screen.height_in_pixels,
+                self.width,
+                self.height,
             )
 
         for depth in self.default_screen.allowed_depths:
@@ -719,13 +672,63 @@ class Painter:
                     break
 
         surface = cairocffi.xcb.XCBSurface(
-            self.conn,
-            root_pixmap,
-            root_visual,
-            self.default_screen.width_in_pixels,
-            self.default_screen.height_in_pixels,
+            self.conn, root_pixmap, root_visual, self.width, self.height
         )
 
+        return root_pixmap, surface
+
+    def _update_root_pixmap(self, root_pixmap):
+        self.conn.core.ChangeProperty(
+            xcffib.xproto.PropMode.Replace,
+            self.default_screen.root.wid,
+            self.atoms["_XROOTPMAP_ID"],
+            xcffib.xproto.Atom.PIXMAP,
+            32,
+            1,
+            [root_pixmap],
+        )
+        self.conn.core.ChangeProperty(
+            xcffib.xproto.PropMode.Replace,
+            self.default_screen.root.wid,
+            self.atoms["ESETROOT_PMAP_ID"],
+            xcffib.xproto.Atom.PIXMAP,
+            32,
+            1,
+            [root_pixmap],
+        )
+        self.conn.core.ChangeWindowAttributes(
+            self.default_screen.root.wid, CW.BackPixmap, [root_pixmap]
+        )
+        self.conn.core.ClearArea(0, self.default_screen.root.wid, 0, 0, self.width, self.height)
+        self.conn.flush()
+
+        # now that we have drawn the new pixmap, free the old one
+        if self.root_pixmap_id is not None and self.root_pixmap_id != root_pixmap:
+            self.conn.core.FreePixmap(self.root_pixmap_id)
+        self.root_pixmap_id = root_pixmap
+
+    def fill(self, screen, background):
+        root_pixmap, surface = self._get_root_pixmap_and_surface(screen)
+
+        with cairocffi.Context(surface) as ctx:
+            ctx.translate(screen.x, screen.y)
+            ctx.rectangle(0, 0, screen.width, screen.height)
+            ctx.set_source_rgba(*rgb(background))
+            ctx.fill()
+
+        surface.finish()
+        self._update_root_pixmap(root_pixmap)
+
+    def paint(self, screen, image_path, mode=None):
+        try:
+            with open(image_path, "rb") as f:
+                image, _ = cairocffi.pixbuf.decode_to_image_surface(f.read())
+        except OSError:
+            logger.exception("Could not load wallpaper:")
+            return
+
+        root_pixmap, surface = self._get_root_pixmap_and_surface(screen)
+
         context = cairocffi.Context(surface)
         with context:
             context.translate(screen.x, screen.y)
@@ -749,49 +752,21 @@ class Painter:
             context.set_source_surface(image)
             context.paint()
 
-        self.conn.core.ChangeProperty(
-            xcffib.xproto.PropMode.Replace,
-            self.default_screen.root.wid,
-            self.atoms["_XROOTPMAP_ID"],
-            xcffib.xproto.Atom.PIXMAP,
-            32,
-            1,
-            [root_pixmap],
-        )
-        self.conn.core.ChangeProperty(
-            xcffib.xproto.PropMode.Replace,
-            self.default_screen.root.wid,
-            self.atoms["ESETROOT_PMAP_ID"],
-            xcffib.xproto.Atom.PIXMAP,
-            32,
-            1,
-            [root_pixmap],
-        )
-        self.conn.core.ChangeWindowAttributes(
-            self.default_screen.root.wid, CW.BackPixmap, [root_pixmap]
-        )
-        self.conn.core.ClearArea(
-            0,
-            self.default_screen.root.wid,
-            0,
-            0,
-            self.default_screen.width_in_pixels,
-            self.default_screen.height_in_pixels,
-        )
-        self.conn.flush()
+        surface.finish()
+        self._update_root_pixmap(root_pixmap)
 
     def __del__(self):
         self.conn.disconnect()
 
 
 def get_keysym(key: str) -> int:
-    keysym = keysyms.get(key)
+    keysym = keysyms.get(key.lower())
     if not keysym:
-        raise XCBQError("Unknown key: %s" % key)
+        raise XCBQError(f"Unknown key: {key}")
     return keysym
 
 
-def translate_modifiers(mask: int) -> typing.List[str]:
+def translate_modifiers(mask: int) -> list[str]:
     r = []
     for k, v in ModMasks.items():
         if mask & v:
@@ -799,7 +774,7 @@ def translate_modifiers(mask: int) -> typing.List[str]:
     return r
 
 
-def translate_masks(modifiers: typing.List[str]) -> int:
+def translate_masks(modifiers: list[str]) -> int:
     """
     Translate a modifier mask specified as a list of strings into an or-ed
     bit representation.
@@ -807,9 +782,9 @@ def translate_masks(modifiers: typing.List[str]) -> int:
     masks = []
     for i in modifiers:
         try:
-            masks.append(ModMasks[i])
+            masks.append(ModMasks[i.lower()])
         except KeyError as e:
-            raise XCBQError("Unknown modifier: %s" % i) from e
+            raise XCBQError(f"Unknown modifier: {i}") from e
     if masks:
         return functools.reduce(operator.or_, masks)
     else:
