@@ -28,12 +28,9 @@ import sys
 from pathlib import Path
 from typing import TYPE_CHECKING
 
-from libqtile.backend.x11 import core
-
 if TYPE_CHECKING:
-    from typing import Any, Dict, List, Union
-
-    from typing_extensions import Literal
+    from types import FunctionType
+    from typing import Any, Literal
 
     from libqtile.config import Group, Key, Mouse, Rule, Screen
     from libqtile.layout.base import Layout
@@ -44,8 +41,8 @@ class ConfigError(Exception):
 
 
 config_pyi_header = """
-from typing import Any, Dict, List, Union
-from typing_extensions import Literal
+from typing import Any
+from typing import Literal
 from libqtile.config import Group, Key, Mouse, Rule, Screen
 from libqtile.layout.base import Layout
 
@@ -54,24 +51,30 @@ from libqtile.layout.base import Layout
 
 class Config:
     # All configuration options
-    keys: List[Key]
-    mouse: List[Mouse]
-    groups: List[Group]
+    keys: list[Key]
+    mouse: list[Mouse]
+    groups: list[Group]
     dgroups_key_binder: Any
-    dgroups_app_rules: List[Rule]
-    follow_mouse_focus: bool
-    focus_on_window_activation: Literal["focus", "smart", "urgent", "never"]
+    dgroups_app_rules: list[Rule]
+    follow_mouse_focus: bool | Literal["click_or_drag_only"]
+    focus_on_window_activation: Literal["focus", "smart", "urgent", "never"] | FunctionType
     cursor_warp: bool
-    layouts: List[Layout]
+    layouts: list[Layout]
     floating_layout: Layout
-    screens: List[Screen]
+    screens: list[Screen]
     auto_fullscreen: bool
-    widget_defaults: Dict[str, Any]
-    extension_defaults: Dict[str, Any]
-    bring_front_click: Union[bool, Literal["floating_only"]]
+    widget_defaults: dict[str, Any]
+    extension_defaults: dict[str, Any]
+    bring_front_click: bool | Literal["floating_only"]
+    floats_kept_above: bool
     reconfigure_screens: bool
     wmname: str
     auto_minimize: bool
+    # Really we'd want to check this Any is libqtile.backend.wayland.ImportConfig, but
+    # doing so forces the import, creating a hard dependency for wlroots.
+    wl_input_rules: dict[str, Any] | None
+    wl_xcursor_theme: str | None
+    wl_xcursor_size: int
 
     def __init__(self, file_path=None, **settings):
         """Create a Config() object from settings
@@ -96,6 +99,27 @@ class Config:
                 value = getattr(self, key, default[key])
             setattr(self, key, value)
 
+    def _reload_config_submodules(self, path: Path) -> None:
+        """Reloads python files from same folder as config file."""
+        folder = path.parent
+        for module in sys.modules.copy().values():
+            # Skip built-ins and anything with no filepath.
+            if hasattr(module, "__file__") and module.__file__ is not None:
+                subpath = Path(module.__file__)
+
+                if subpath == path:
+                    # do not reevaluate config itself here, we want only
+                    # reload all submodules. Also we cant reevaluate config
+                    # here, because it will cache all current modules before they
+                    # are reloaded. Thus, config file should be reloaded after
+                    # this routine.
+                    continue
+
+                # Check if the module is in the config folder or subfolder
+                # and the file still exists.  If so, reload it
+                if folder in subpath.parents and subpath.exists():
+                    importlib.reload(module)
+
     def load(self):
         if not self.file_path:
             return
@@ -105,6 +129,7 @@ class Config:
         sys.path.insert(0, path.parent.as_posix())
 
         if name in sys.modules:
+            self._reload_config_submodules(path)
             config = importlib.reload(sys.modules[name])
         else:
             config = importlib.import_module(name)
@@ -113,20 +138,25 @@ class Config:
 
     def validate(self) -> None:
         """
-        Validate the configuration against the core.
+        Validate the configuration against the X11 core, if it makes sense.
         """
+        try:
+            from libqtile.backend.x11 import core
+        except ImportError:
+            return
+
         valid_keys = core.get_keys()
         valid_mods = core.get_modifiers()
         # we explicitly do not want to set self.keys and self.mouse above,
         # because they are dynamically resolved from the default_config. so we
         # need to ignore the errors here about missing attributes.
         for k in self.keys:
-            if k.key not in valid_keys:
-                raise ConfigError("No such key: %s" % k.key)
+            if isinstance(k.key, str) and k.key.lower() not in valid_keys:
+                raise ConfigError(f"No such key: {k.key}")
             for m in k.modifiers:
-                if m not in valid_mods:
-                    raise ConfigError("No such modifier: %s" % m)
+                if m.lower() not in valid_mods:
+                    raise ConfigError(f"No such modifier: {m}")
         for ms in self.mouse:
             for m in ms.modifiers:
-                if m not in valid_mods:
-                    raise ConfigError("No such modifier: %s" % m)
+                if m.lower() not in valid_mods:
+                    raise ConfigError(f"No such modifier: {m}")
