@@ -26,23 +26,22 @@
 # SOFTWARE.
 
 import collections
-import six
 
 import libqtile.hook
-from libqtile.config import Key
-from libqtile.command import lazy
-from libqtile.config import Group
-from libqtile.config import Rule
-from libqtile.config import Match
+from libqtile.backend.base import Static
+from libqtile.config import Group, Key, Match, Rule
+from libqtile.lazy import lazy
 from libqtile.log_utils import logger
 
 
 def simple_key_binder(mod, keynames=None):
     """Bind keys to mod+group position or to the keys specified as second argument"""
+
     def func(dgroup):
         # unbind all
         for key in dgroup.keys[:]:
-            dgroup.qtile.unmapKey(key)
+            dgroup.qtile.ungrab_key(key)
+            dgroup.qtile.config.keys.remove(key)
             dgroup.keys.remove(key)
 
         if keynames:
@@ -56,34 +55,30 @@ def simple_key_binder(mod, keynames=None):
             name = group.name
             key = Key([mod], keyname, lazy.group[name].toscreen())
             key_s = Key([mod, "shift"], keyname, lazy.window.togroup(name))
-            key_c = Key(
-                [mod, "control"],
-                keyname,
-                lazy.group.switch_groups(name)
-            )
-            dgroup.keys.append(key)
-            dgroup.keys.append(key_s)
-            dgroup.keys.append(key_c)
-            dgroup.qtile.mapKey(key)
-            dgroup.qtile.mapKey(key_s)
-            dgroup.qtile.mapKey(key_c)
+            key_c = Key([mod, "control"], keyname, lazy.group.switch_groups(name))
+            dgroup.keys.extend([key, key_s, key_c])
+            dgroup.qtile.config.keys.extend([key, key_s, key_c])
+            dgroup.qtile.grab_key(key)
+            dgroup.qtile.grab_key(key_s)
+            dgroup.qtile.grab_key(key_c)
 
     return func
 
 
-class DGroups(object):
+class DGroups:
     """Dynamic Groups"""
+
     def __init__(self, qtile, dgroups, key_binder=None, delay=1):
         self.qtile = qtile
 
         self.groups = dgroups
-        self.groupMap = {}
+        self.groups_map = {}
 
         self.rules = []
         self.rules_map = {}
         self.last_rule_id = 0
 
-        for rule in getattr(qtile.config, 'dgroups_app_rules', []):
+        for rule in getattr(qtile.config, "dgroups_app_rules", []):
             self.add_rule(rule)
 
         self.keys = []
@@ -113,51 +108,53 @@ class DGroups(object):
             self.rules.remove(rule)
             del self.rules_map[rule_id]
         else:
-            logger.warn('Rule "%s" not found', rule_id)
+            logger.warning('Rule "%s" not found', rule_id)
 
     def add_dgroup(self, group, start=False):
-        self.groupMap[group.name] = group
-        rules = [Rule(m, group=group.name) for m in group.matches]
-        self.rules.extend(rules)
+        self.groups_map[group.name] = group
+        rule = Rule(group.matches, group=group.name)
+        self.rules.append(rule)
         if start:
-            self.qtile.addGroup(group.name, group.layout, group.layouts)
+            self.qtile.add_group(
+                group.name,
+                group.layout,
+                group.layouts,
+                group.label,
+                screen_affinity=group.screen_affinity,
+            )
 
     def _setup_groups(self):
         for group in self.groups:
             self.add_dgroup(group, group.init)
 
             if group.spawn and not self.qtile.no_spawn:
-                if isinstance(group.spawn, six.string_types):
+                if isinstance(group.spawn, str):
                     spawns = [group.spawn]
                 else:
                     spawns = group.spawn
                 for spawn in spawns:
-                    pid = self.qtile.cmd_spawn(spawn)
-                    self.add_rule(Rule(Match(net_wm_pid=[pid]), group.name))
+                    pid = self.qtile.spawn(spawn)
+                    self.add_rule(Rule(Match(net_wm_pid=pid), group.name))
 
     def _setup_hooks(self):
         libqtile.hook.subscribe.addgroup(self._addgroup)
         libqtile.hook.subscribe.client_new(self._add)
         libqtile.hook.subscribe.client_killed(self._del)
         if self.key_binder:
-            libqtile.hook.subscribe.setgroup(
-                lambda: self.key_binder(self)
-            )
-            libqtile.hook.subscribe.changegroup(
-                lambda: self.key_binder(self)
-            )
+            libqtile.hook.subscribe.setgroup(lambda: self.key_binder(self))
+            libqtile.hook.subscribe.changegroup(lambda: self.key_binder(self))
 
-    def _addgroup(self, qtile, group_name):
-        if group_name not in self.groupMap:
+    def _addgroup(self, group_name):
+        if group_name not in self.groups_map:
             self.add_dgroup(Group(group_name, persist=False))
 
     def _add(self, client):
         if client in self.timeout:
-            logger.info('Remove dgroup source')
+            logger.debug("Remove dgroup source")
             self.timeout.pop(client).cancel()
 
         # ignore static windows
-        if client.defunct:
+        if isinstance(client, Static):
             return
 
         # ignore windows whose groups is already set (e.g. from another hook or
@@ -172,33 +169,33 @@ class DGroups(object):
             # Matching Rules
             if rule.matches(client):
                 if rule.group:
-                    try:
-                        layout = self.groupMap[rule.group].layout
-                    except KeyError:
+                    if rule.group in self.groups_map:
+                        layout = self.groups_map[rule.group].layout
+                        layouts = self.groups_map[rule.group].layouts
+                        label = self.groups_map[rule.group].label
+                    else:
                         layout = None
-                    try:
-                        layouts = self.groupMap[rule.group].layouts
-                    except KeyError:
                         layouts = None
-                    group_added = self.qtile.addGroup(rule.group, layout, layouts)
+                        label = None
+                    group_added = self.qtile.add_group(rule.group, layout, layouts, label)
                     client.togroup(rule.group)
 
                     group_set = True
 
-                    group_obj = self.qtile.groupMap[rule.group]
-                    group = self.groupMap.get(rule.group)
+                    group_obj = self.qtile.groups_map[rule.group]
+                    group = self.groups_map.get(rule.group)
                     if group and group_added:
                         for k, v in list(group.layout_opts.items()):
-                            if isinstance(v, collections.Callable):
+                            if isinstance(v, collections.abc.Callable):
                                 v(group_obj.layout)
                             else:
                                 setattr(group_obj.layout, k, v)
                         affinity = group.screen_affinity
                         if affinity and len(self.qtile.screens) > affinity:
-                            self.qtile.screens[affinity].setGroup(group_obj)
+                            self.qtile.screens[affinity].set_group(group_obj)
 
                 if rule.float:
-                    client.enablefloating()
+                    client.enable_floating()
 
                 if rule.intrusive:
                     intrusive = rule.intrusive
@@ -208,12 +205,13 @@ class DGroups(object):
 
         # If app doesn't have a group
         if not group_set:
-            current_group = self.qtile.currentGroup.name
-            if current_group in self.groupMap and \
-                    self.groupMap[current_group].exclusive and \
-                    not intrusive:
-
-                wm_class = client.window.get_wm_class()
+            current_group = self.qtile.current_group.name
+            if (
+                current_group in self.groups_map
+                and self.groups_map[current_group].exclusive
+                and not intrusive
+            ):
+                wm_class = client.get_wm_class()
 
                 if wm_class:
                     if len(wm_class) > 1:
@@ -223,30 +221,41 @@ class DGroups(object):
 
                     group_name = wm_class
                 else:
-                    group_name = client.name or 'Unnamed'
+                    group_name = client.name or "Unnamed"
 
                 self.add_dgroup(Group(group_name, persist=False), start=True)
                 client.togroup(group_name)
         self.sort_groups()
 
     def sort_groups(self):
-        self.qtile.groups.sort(key=lambda g: self.groupMap[g.name].position)
-        libqtile.hook.fire("setgroup")
+        grps = self.qtile.groups
+        sorted_grps = sorted(grps, key=lambda g: self.groups_map[g.name].position)
+        if grps != sorted_grps:
+            self.qtile.groups = sorted_grps
+            libqtile.hook.fire("changegroup")
 
     def _del(self, client):
+        # ignore static windows
+        if isinstance(client, Static):
+            return
+
         group = client.group
 
         def delete_client():
             # Delete group if empty and don't persist
-            if group and group.name in self.groupMap and \
-                    not self.groupMap[group.name].persist and \
-                    len(group.windows) <= 0:
-                self.qtile.delGroup(group.name)
+            if (
+                group
+                and group.name in self.groups_map
+                and not self.groups_map[group.name].persist
+                and len(group.windows) <= 0
+            ):
+                self.qtile.delete_group(group.name)
                 self.sort_groups()
             del self.timeout[client]
 
-        # Wait the delay until really delete the group
-        logger.info('Add dgroup timer')
-        self.timeout[client] = self.qtile.call_later(
-            self.delay, delete_client
-        )
+        if group is not None and group.persist:
+            return
+
+        logger.debug("Deleting %s in %ss", group, self.delay)
+        if client not in self.timeout:
+            self.timeout[client] = self.qtile.call_later(self.delay, delete_client)
