@@ -26,11 +26,17 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from libqtile import hook, utils
-from libqtile.backend.base import FloatStates
-from libqtile.command.base import CommandObject, ItemT
+from libqtile.command.base import CommandObject, expose_command
 from libqtile.log_utils import logger
 
+if TYPE_CHECKING:
+    from libqtile.command.base import ItemT
+
 
 class _Group(CommandObject):
     """A container for a bunch of windows
@@ -41,7 +47,8 @@ class _Group(CommandObject):
     A group is identified by its name but displayed in GroupBox widget by its label.
     """
 
-    def __init__(self, name, layout=None, label=None):
+    def __init__(self, name, layout=None, label=None, screen_affinity=None, persist=False):
+        self.screen_affinity = screen_affinity
         self.name = name
         self.label = name if label is None else label
         self.custom_layout = layout  # will be set on _configure
@@ -59,6 +66,7 @@ class _Group(CommandObject):
         self.screen = None
         self.current_layout = None
         self.last_focused = None
+        self.persist = persist
 
     def _configure(self, layouts, floating_layout, qtile):
         self.screen = None
@@ -114,20 +122,19 @@ class _Group(CommandObject):
         """
         for index, obj in enumerate(self.layouts):
             if obj.name == layout:
-                self.current_layout = index
-                hook.fire("layout_change", self.layouts[self.current_layout], self)
-                self.layout_all()
+                self.use_layout(index)
                 return
-        logger.error("No such layout: {}".format(layout))
+        logger.error("No such layout: %s", layout)
 
-    def use_layout(self, index):
+    def use_layout(self, index: int):
         assert -len(self.layouts) <= index < len(self.layouts), "layout index out of bounds"
         self.layout.hide()
         self.current_layout = index % len(self.layouts)
         hook.fire("layout_change", self.layouts[self.current_layout], self)
         self.layout_all()
-        screen_rect = self.screen.get_rect()
-        self.layout.show(screen_rect)
+        if self.screen is not None:
+            screen_rect = self.screen.get_rect()
+            self.layout.show(screen_rect)
 
     def use_next_layout(self):
         self.use_layout((self.current_layout + 1) % (len(self.layouts)))
@@ -135,11 +142,14 @@ class _Group(CommandObject):
     def use_previous_layout(self):
         self.use_layout((self.current_layout - 1) % (len(self.layouts)))
 
-    def layout_all(self, warp=False):
+    def layout_all(self, warp=False, focus=True):
         """Layout the floating layer, then the current layout.
 
-        If we have have a current_window give it focus, optionally moving warp
-        to it.
+        Parameters
+        ==========
+        focus :
+            If we have have a current_window give it focus, optionally moving warp
+            to it.
         """
         if self.screen and self.windows:
             with self.qtile.core.masked():
@@ -153,12 +163,13 @@ class _Group(CommandObject):
                         logger.exception("Exception in layout %s", self.layout.name)
                 if floating:
                     self.floating_layout.layout(floating, screen_rect)
-                if self.current_window and self.screen == self.qtile.current_screen:
-                    self.current_window.focus(warp)
-                else:
-                    # Screen has lost focus so we reset record of focused window so
-                    # focus will warp when screen is focused again
-                    self.last_focused = None
+                if focus:
+                    if self.current_window and self.screen == self.qtile.current_screen:
+                        self.current_window.focus(warp)
+                    else:
+                        # Screen has lost focus so we reset record of focused window so
+                        # focus will warp when screen is focused again
+                        self.last_focused = None
 
     def set_screen(self, screen, warp=True):
         """Set this group's screen to screen"""
@@ -224,7 +235,9 @@ class _Group(CommandObject):
             hook.fire("focus_change")
             self.layout_all(warp)
 
+    @expose_command()
     def info(self):
+        """Returns a dictionary of info for this group"""
         return dict(
             name=self.name,
             label=self.label,
@@ -244,19 +257,22 @@ class _Group(CommandObject):
             self.windows.append(win)
         win.group = self
         if self.qtile.config.auto_fullscreen and win.wants_to_fullscreen:
-            win._float_state = FloatStates.FULLSCREEN
-        elif self.floating_layout.match(win):
-            win._float_state = FloatStates.FLOATING
-        if win.floating:
-            self.floating_layout.add(win)
-        if not win.floating or win.fullscreen:
+            win.fullscreen = True
+        elif self.floating_layout.match(win) and not win.fullscreen:
+            win.floating = True
+        if win.floating and not win.fullscreen:
+            self.floating_layout.add_client(win)
+        else:
             self.tiled_windows.add(win)
             for i in self.layouts:
-                i.add(win)
+                i.add_client(win)
         if focus:
             self.focus(win, warp=True, force=force)
+        else:
+            self.layout_all(focus=False)
 
     def remove(self, win, force=False):
+        hook.fire("group_window_remove", self, win)
         self.windows.remove(win)
         hadfocus = self._remove_from_focus_history(win)
         win.group = None
@@ -285,7 +301,8 @@ class _Group(CommandObject):
                 or self.layout.focus_first()
             )
 
-            self.tiled_windows.remove(win)
+            if win in self.tiled_windows:
+                self.tiled_windows.remove(win)
 
         # a notification may not have focus
         if hadfocus:
@@ -310,16 +327,16 @@ class _Group(CommandObject):
                         i.remove(win)
                         if win is self.current_window:
                             i.blur()
-                self.floating_layout.add(win)
-                if win is self.current_window:
-                    self.floating_layout.focus(win)
+                    self.floating_layout.add_client(win)
+                    if win is self.current_window:
+                        self.floating_layout.focus(win)
         else:
             self.floating_layout.remove(win)
             self.floating_layout.blur()
             # A window that was fullscreen should only be added if it was not a tiled window
             if win not in self.tiled_windows:
                 for i in self.layouts:
-                    i.add(win)
+                    i.add_client(win)
                 self.tiled_windows.add(win)
             if win is self.current_window:
                 for i in self.layouts:
@@ -348,16 +365,14 @@ class _Group(CommandObject):
             for i in self.windows:
                 if i.wid == sel:
                     return i
-        raise RuntimeError("Invalid selection: {}".format(name))
+        raise RuntimeError(f"Invalid selection: {name}")
 
-    def cmd_setlayout(self, layout):
+    @expose_command()
+    def setlayout(self, layout):
         self.layout = layout
 
-    def cmd_info(self):
-        """Returns a dictionary of info for this group"""
-        return self.info()
-
-    def cmd_toscreen(self, screen=None, toggle=False):
+    @expose_command()
+    def toscreen(self, screen=None, toggle=False):
         """Pull a group to a specified screen.
 
         Parameters
@@ -413,9 +428,13 @@ class _Group(CommandObject):
                 return False
             return True
 
-        groups = [group for group in self.qtile.groups if match(group)]
-        index = (groups.index(self) + direction) % len(groups)
-        return groups[index]
+        try:
+            groups = [group for group in self.qtile.groups if match(group)]
+            index = (groups.index(self) + direction) % len(groups)
+            return groups[index]
+        except ValueError:
+            # group is not managed
+            return None
 
     def get_previous_group(self, skip_empty=False, skip_managed=False):
         return self._get_group(-1, skip_empty, skip_managed)
@@ -423,13 +442,15 @@ class _Group(CommandObject):
     def get_next_group(self, skip_empty=False, skip_managed=False):
         return self._get_group(1, skip_empty, skip_managed)
 
-    def cmd_unminimize_all(self):
+    @expose_command()
+    def unminimize_all(self):
         """Unminimise all windows in this group"""
         for win in self.windows:
             win.minimized = False
         self.layout_all()
 
-    def cmd_next_window(self):
+    @expose_command()
+    def next_window(self):
         """
         Focus the next window in group.
 
@@ -453,7 +474,8 @@ class _Group(CommandObject):
             )
         self.focus(nxt, True)
 
-    def cmd_prev_window(self):
+    @expose_command()
+    def prev_window(self):
         """
         Focus the previous window in group.
 
@@ -477,7 +499,8 @@ class _Group(CommandObject):
             )
         self.focus(nxt, True)
 
-    def cmd_focus_back(self):
+    @expose_command()
+    def focus_back(self):
         """
         Focus the window that had focus before the current one got it.
 
@@ -492,7 +515,8 @@ class _Group(CommandObject):
         else:
             self.focus(win)
 
-    def cmd_focus_by_name(self, name):
+    @expose_command()
+    def focus_by_name(self, name):
         """
         Focus the first window with the given name. Do nothing if the name is
         not found.
@@ -502,7 +526,8 @@ class _Group(CommandObject):
                 self.focus(win)
                 break
 
-    def cmd_info_by_name(self, name):
+    @expose_command()
+    def info_by_name(self, name):
         """
         Get the info for the first window with the given name without giving it
         focus. Do nothing if the name is not found.
@@ -511,11 +536,40 @@ class _Group(CommandObject):
             if win.name == name:
                 return win.info()
 
-    def cmd_switch_groups(self, name):
-        """Switch position of current group with name"""
-        self.qtile.cmd_switch_groups(self.name, name)
+    @expose_command()
+    def focus_by_index(self, index: int) -> None:
+        """
+        Change to the window at the specified index in the current group.
+        """
+        windows = self.windows
+        if index < 0 or index > len(windows) - 1:
+            return
 
-    def cmd_set_label(self, label):
+        self.focus(windows[index])
+
+    @expose_command()
+    def swap_window_order(self, new_location: int) -> None:
+        """
+        Change the order of the current window within the current group.
+        """
+        if new_location < 0 or new_location > len(self.windows) - 1:
+            return
+
+        windows = self.windows
+        current_window_index = windows.index(self.current_window)
+
+        windows[current_window_index], windows[new_location] = (
+            windows[new_location],
+            windows[current_window_index],
+        )
+
+    @expose_command()
+    def switch_groups(self, name):
+        """Switch position of current group with name"""
+        self.qtile.switch_groups(self.name, name)
+
+    @expose_command()
+    def set_label(self, label):
         """
         Set the display name of current group to be used in GroupBox widget.
         If label is None, the name of the group is used as display name.
@@ -525,4 +579,4 @@ class _Group(CommandObject):
         hook.fire("changegroup")
 
     def __repr__(self):
-        return "<group.Group (%r)>" % self.name
+        return f"<group.Group ({self.name!r})>"
