@@ -27,20 +27,27 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
 
-from typing import List, Optional
+from typing import TYPE_CHECKING
 
-from libqtile.backend.base import Window
-from libqtile.config import Match
+from libqtile.command.base import expose_command
+from libqtile.config import Match, _Match
 from libqtile.layout.base import Layout
 
+if TYPE_CHECKING:
+    from typing import Any
+
+    from libqtile.backend.base import Window
+    from libqtile.config import ScreenRect
+
 
 class Floating(Layout):
     """
     Floating layout, which does nothing with windows but handles focus order
     """
 
-    default_float_rules = [
+    default_float_rules: list[_Match] = [
         Match(wm_type="utility"),
         Match(wm_type="notification"),
         Match(wm_type="toolbar"),
@@ -67,7 +74,7 @@ class Floating(Layout):
     ]
 
     def __init__(
-        self, float_rules: Optional[List[Match]] = None, no_reposition_rules=None, **config
+        self, float_rules: list[_Match] | None = None, no_reposition_rules=None, **config
     ):
         """
         If you have certain apps that you always want to float you can provide
@@ -98,9 +105,8 @@ class Floating(Layout):
         correct location on the screen.
         """
         Layout.__init__(self, **config)
-        self.clients: List[Window] = []
-        self.focused = None
-        self.group = None
+        self.clients: list[Window] = []
+        self.focused: Window | None = None
 
         if float_rules is None:
             float_rules = self.default_float_rules
@@ -151,14 +157,15 @@ class Floating(Layout):
         if clients:
             return clients[0]
 
-    def focus_next(self, win):
+    def focus_next(self, win: Window) -> Window | None:
         if win not in self.clients or win.group is None:
-            return
+            return None
 
         clients = self.find_clients(win.group)
         idx = clients.index(win)
         if len(clients) > idx + 1:
             return clients[idx + 1]
+        return None
 
     def focus_last(self, group=None):
         if group is None:
@@ -178,10 +185,10 @@ class Floating(Layout):
         if idx > 0:
             return clients[idx - 1]
 
-    def focus(self, client):
+    def focus(self, client: Window) -> None:
         self.focused = client
 
-    def blur(self):
+    def blur(self) -> None:
         self.focused = None
 
     def on_screen(self, client, screen_rect):
@@ -232,7 +239,7 @@ class Floating(Layout):
             client.y = int(round(y))
         return above
 
-    def configure(self, client, screen_rect):
+    def configure(self, client: Window, screen_rect: ScreenRect) -> None:
         if client.has_focus:
             bc = self.border_focus
         else:
@@ -251,12 +258,12 @@ class Floating(Layout):
         is_java_dropdown = "sun-awt-X11-XWindowPeer" in cls
         if is_java_dropdown:
             client.paint_borders(bc, bw)
-            client.cmd_bring_to_front()
+            client.bring_to_front()
 
         # alternatively, users may have asked us explicitly to leave the client alone
         elif any(m.compare(client) for m in self.no_reposition_rules):
             client.paint_borders(bc, bw)
-            client.cmd_bring_to_front()
+            client.bring_to_front()
 
         else:
             above = False
@@ -278,13 +285,13 @@ class Floating(Layout):
             )
         client.unhide()
 
-    def add(self, client):
+    def add_client(self, client: Window) -> None:
         self.clients.append(client)
         self.focused = client
 
-    def remove(self, client):
+    def remove(self, client: Window) -> Window | None:
         if client not in self.clients:
-            return
+            return None
 
         next_focus = self.focus_next(client)
         if client is self.focused:
@@ -295,15 +302,20 @@ class Floating(Layout):
     def get_windows(self):
         return self.clients
 
-    def info(self):
-        d = Layout.info(self)
-        d["clients"] = [c.name for c in self.clients]
+    @expose_command()
+    def info(self) -> dict[str, Any]:
+        d = dict(
+            name=self.name,
+            clients=[c.name for c in self.clients],
+        )
         return d
 
-    def cmd_next(self):
+    @expose_command()
+    def next(self) -> None:
         # This can't ever be called, but implement the abstract method
         pass
 
-    def cmd_previous(self):
+    @expose_command()
+    def previous(self) -> None:
         # This can't ever be called, but implement the abstract method
         pass
