@@ -29,10 +29,21 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
 
-from libqtile.config import Match
+from typing import TYPE_CHECKING
+
+from libqtile.command.base import expose_command
+from libqtile.config import _Match
 from libqtile.layout.base import _SimpleLayoutBase
 
+if TYPE_CHECKING:
+    from typing import Any, Self
+
+    from libqtile.backend.base import Window
+    from libqtile.config import ScreenRect
+    from libqtile.group import _Group
+
 
 class Tile(_SimpleLayoutBase):
     """A layout with two stacks of windows dividing the screen
@@ -69,8 +80,8 @@ class Tile(_SimpleLayoutBase):
         (
             "ratio_increment",
             0.05,
-            "By which amount to change ratio when cmd_decrease_ratio or "
-            "cmd_increase_ratio are called.",
+            "By which amount to change ratio when decrease_ratio or "
+            "increase_ratio are called.",
         ),
         (
             "add_on_top",
@@ -118,14 +129,16 @@ class Tile(_SimpleLayoutBase):
     def slave_windows(self):
         return self.clients[self.master_length :]
 
-    def up(self):
+    @expose_command("shuffle_left")
+    def shuffle_up(self):
         if self.shift_windows:
             self.clients.shuffle_up()
         else:
             self.clients.rotate_down()
         self.group.layout_all()
 
-    def down(self):
+    @expose_command("shuffle_right")
+    def shuffle_down(self):
         if self.shift_windows:
             self.clients.shuffle_down()
         else:
@@ -137,7 +150,7 @@ class Tile(_SimpleLayoutBase):
             return
         if self.clients:
             master_match = match or self.master_match
-            if isinstance(master_match, Match):
+            if isinstance(master_match, _Match):
                 master_match = [master_match]
             masters = []
             for c in self.clients:
@@ -148,20 +161,20 @@ class Tile(_SimpleLayoutBase):
                 self.clients.remove(client)
                 self.clients.append_head(client)
 
-    def clone(self, group):
+    def clone(self, group: _Group) -> Self:
         c = _SimpleLayoutBase.clone(self, group)
         return c
 
-    def add(self, client, offset_to_current=1):
+    def add_client(self, client, offset_to_current=1):
         if self.add_after_last:
             self.clients.append(client)
         elif self.add_on_top:
             self.clients.append_head(client)
         else:
-            super().add(client, offset_to_current)
+            super().add_client(client, offset_to_current)
         self.reset_master()
 
-    def configure(self, client, screen_rect):
+    def configure(self, client: Window, screen_rect: ScreenRect) -> None:
         screen_width = screen_rect.width
         screen_height = screen_rect.height
         border_width = self.border_width
@@ -180,7 +193,10 @@ class Tile(_SimpleLayoutBase):
                 w = screen_width - int(screen_width * self.ratio_size)
                 h = screen_height // (len(self.slave_windows))
                 x = screen_rect.x + int(screen_width * self.ratio_size)
-                y = screen_rect.y + self.clients[self.master_length :].index(client) * h
+                sublist = self.clients[self.master_length :]
+                if client not in sublist:
+                    raise ValueError("Client not in layout. This shouldn't happen.")
+                y = screen_rect.y + sublist.index(client) * h
             if client.has_focus:
                 bc = self.border_focus
             else:
@@ -204,7 +220,8 @@ class Tile(_SimpleLayoutBase):
         else:
             client.hide()
 
-    def info(self):
+    @expose_command()
+    def info(self) -> dict[str, Any]:
         d = _SimpleLayoutBase.info(self)
         d.update(
             dict(
@@ -214,42 +231,37 @@ class Tile(_SimpleLayoutBase):
         )
         return d
 
-    def cmd_shuffle_down(self):
-        self.down()
+    @expose_command(["left", "up"])
+    def previous(self) -> None:
+        _SimpleLayoutBase.previous(self)
 
-    def cmd_shuffle_up(self):
-        self.up()
+    @expose_command(["right", "down"])
+    def next(self) -> None:
+        _SimpleLayoutBase.next(self)
 
-    def cmd_reset(self):
+    @expose_command("normalize")
+    def reset(self):
         self.ratio_size = self._initial_ratio
         self.group.layout_all()
 
-    cmd_normalize = cmd_reset
-
-    cmd_shuffle_left = cmd_shuffle_up
-    cmd_shuffle_right = cmd_shuffle_down
-
-    cmd_previous = _SimpleLayoutBase.previous
-    cmd_next = _SimpleLayoutBase.next
-    cmd_up = cmd_previous
-    cmd_down = cmd_next
-    cmd_left = cmd_previous
-    cmd_right = cmd_next
-
-    def cmd_decrease_ratio(self):
+    @expose_command()
+    def decrease_ratio(self):
         self.ratio_size -= self.ratio_increment
         self.group.layout_all()
 
-    def cmd_increase_ratio(self):
+    @expose_command()
+    def increase_ratio(self):
         self.ratio_size += self.ratio_increment
         self.group.layout_all()
 
-    def cmd_decrease_nmaster(self):
+    @expose_command()
+    def decrease_nmaster(self):
         self.master_length -= 1
         if self.master_length <= 0:
             self.master_length = 1
         self.group.layout_all()
 
-    def cmd_increase_nmaster(self):
+    @expose_command()
+    def increase_nmaster(self):
         self.master_length += 1
         self.group.layout_all()
