@@ -46,51 +46,48 @@
 #
 # This is not intended to be a complete cffi-based pango binding.
 
-import six
 
-try:
-    from libqtile._ffi_pango import ffi
-except ImportError:
-    # PyPy < 2.6 (cffi < 1) compatibility
-    import cffi
-    if cffi.__version_info__[0] == 0:
-        from libqtile.ffi_build import pango_ffi as ffi
-    else:
-        raise ImportError("No module named libqtile._ffi_pango, be sure to run `python ./libqtile/ffi_build.py`")
+from libqtile.pango_ffi import pango_ffi as ffi
 
-gobject = ffi.dlopen('libgobject-2.0.so.0')
-pango = ffi.dlopen('libpango-1.0.so.0')
-pangocairo = ffi.dlopen('libpangocairo-1.0.so.0')
+gobject = ffi.dlopen("libgobject-2.0.so.0")  # type: ignore
+pango = ffi.dlopen("libpango-1.0.so.0")  # type: ignore
+pangocairo = ffi.dlopen("libpangocairo-1.0.so.0")  # type: ignore
 
 
-def CairoContext(cairo_t):
+def patch_cairo_context(cairo_t):
     def create_layout():
         return PangoLayout(cairo_t._pointer)
+
     cairo_t.create_layout = create_layout
 
     def show_layout(layout):
         pangocairo.pango_cairo_show_layout(cairo_t._pointer, layout._pointer)
+
     cairo_t.show_layout = show_layout
 
     return cairo_t
 
+
 ALIGN_CENTER = pango.PANGO_ALIGN_CENTER
 ELLIPSIZE_END = pango.PANGO_ELLIPSIZE_END
 units_from_double = pango.pango_units_from_double
 
 
-def _const_char_to_py_str(cc):
-    return ''.join(ffi.buffer(cc, len(cc)))
+ALIGNMENTS = {
+    "left": pango.PANGO_ALIGN_LEFT,
+    "center": pango.PANGO_ALIGN_CENTER,
+    "right": pango.PANGO_ALIGN_RIGHT,
+}
 
 
-class PangoLayout(object):
+class PangoLayout:
     def __init__(self, cairo_t):
         self._cairo_t = cairo_t
         self._pointer = pangocairo.pango_cairo_create_layout(cairo_t)
 
         def free(p):
-            p = ffi.cast("gpointer", p)
             gobject.g_object_unref(p)
+
         self._pointer = ffi.gc(self._pointer, free)
 
     def finalize(self):
@@ -98,6 +95,9 @@ class PangoLayout(object):
         self._pointer = None
         self._cairo_t = None
 
+    def finalized(self):
+        return self._pointer is None
+
     def set_font_description(self, desc):
         # save a pointer so it doesn't get GC'd out from under us
         self._desc = desc
@@ -114,12 +114,12 @@ class PangoLayout(object):
         pango.pango_layout_set_attributes(self._pointer, attrs)
 
     def set_text(self, text):
-        text = text.encode('utf-8')
+        text = text.encode("utf-8")
         pango.pango_layout_set_text(self._pointer, text, -1)
 
     def get_text(self):
         ret = pango.pango_layout_get_text(self._pointer)
-        return _const_char_to_py_str(ret)
+        return ffi.string(ret).decode()
 
     def set_ellipsize(self, ellipzize):
         pango.pango_layout_set_ellipsize(self._pointer, ellipzize)
@@ -139,7 +139,7 @@ class PangoLayout(object):
         pango.pango_layout_set_width(self._pointer, width)
 
 
-class FontDescription(object):
+class FontDescription:
     def __init__(self, pointer=None):
         if pointer is None:
             self._pointer = pango.pango_font_description_new()
@@ -158,7 +158,7 @@ class FontDescription(object):
 
     def get_family(self):
         ret = pango.pango_font_description_get_family(self._pointer)
-        return _const_char_to_py_str(ret)
+        return ffi.string(ret).decode()
 
     def set_absolute_size(self, size):
         pango.pango_font_description_set_absolute_size(self._pointer, size)
@@ -166,27 +166,24 @@ class FontDescription(object):
     def set_size(self, size):
         pango.pango_font_description_set_size(self._pointer, size)
 
-    def get_size(self, size):
-        return pango.pango_font_description_get_size(self._pointer, size)
+    def get_size(self):
+        return pango.pango_font_description_get_size(self._pointer)
 
 
 def parse_markup(value, accel_marker=0):
     attr_list = ffi.new("PangoAttrList**")
     text = ffi.new("char**")
     error = ffi.new("GError**")
-    if six.PY3:
-        value = value.encode()
+    value = value.encode()
 
     ret = pango.pango_parse_markup(value, -1, accel_marker, attr_list, text, ffi.NULL, error)
 
     if ret == 0:
-        raise Exception("parse_markup() failed for %s" % value)
+        raise Exception(f"parse_markup() failed for {value}")
 
-    return attr_list[0], ffi.string(text[0]), six.unichr(accel_marker)
+    return attr_list[0], ffi.string(text[0]), chr(accel_marker)
 
 
 def markup_escape_text(text):
-    ret = gobject.g_markup_escape_text(text.encode('utf-8'), -1)
-    if six.PY3:
-        return ffi.string(ret).decode()
-    return ffi.string(ret)
+    ret = gobject.g_markup_escape_text(text.encode("utf-8"), -1)
+    return ffi.string(ret).decode()
