@@ -18,11 +18,17 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from typing import Dict, List
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from libqtile import config, group, hook
 from libqtile.backend.base import FloatStates
-from libqtile.config import Match
+from libqtile.command.base import expose_command
+from libqtile.config import Match, _Match
+
+if TYPE_CHECKING:
+    from libqtile.backend.base import Window
 
 
 class WindowVisibilityToggler:
@@ -36,7 +42,7 @@ class WindowVisibilityToggler:
     By default, the window is also hidden if it loses focus.
     """
 
-    def __init__(self, scratchpad_name, window, on_focus_lost_hide, warp_pointer):
+    def __init__(self, scratchpad_name, window: Window, on_focus_lost_hide, warp_pointer):
         """
         Initiliaze the  WindowVisibilityToggler.
 
@@ -109,7 +115,7 @@ class WindowVisibilityToggler:
             win._float_state = FloatStates.TOP
             # add to group and bring it to front.
             win.togroup()
-            win.cmd_bring_to_front()
+            win.bring_to_front()
             # toggle internal flag of visibility
             self.shown = True
 
@@ -167,6 +173,19 @@ class DropDownToggler(WindowVisibilityToggler):
         self.y = ddconfig.y
         self.width = ddconfig.width
         self.height = ddconfig.height
+        self.border_width = window.qtile.config.floating_layout.border_width
+
+        # add "SKIP_TASKBAR" to _NET_WM_STATE atom (for X11)
+        if window.qtile.core.name == "x11":
+            net_wm_state = list(window.window.get_property("_NET_WM_STATE", "ATOM", unpack=int))
+            skip_taskbar = window.qtile.core.conn.atoms["_NET_WM_STATE_SKIP_TASKBAR"]
+            if net_wm_state:
+                if skip_taskbar not in net_wm_state:
+                    net_wm_state.append(skip_taskbar)
+            else:
+                net_wm_state = [skip_taskbar]
+            window.window.set_property("_NET_WM_STATE", net_wm_state)
+
         # Let's add the window to the scratchpad group.
         window.togroup(scratchpad_name)
         window.opacity = ddconfig.opacity
@@ -196,9 +215,9 @@ class DropDownToggler(WindowVisibilityToggler):
             y = int(screen.dy + self.y * screen.dheight)
             win.float_x = x
             win.float_y = y
-            width = int(screen.dwidth * self.width)
-            height = int(screen.dheight * self.height)
-            win.place(x, y, width, height, win.borderwidth, win.bordercolor, respect_hints=True)
+            width = int(screen.dwidth * self.width) - 2 * self.border_width
+            height = int(screen.dheight * self.height) - 2 * self.border_width
+            win.place(x, y, width, height, self.border_width, win.bordercolor, respect_hints=True)
             # Toggle the dropdown
             WindowVisibilityToggler.show(self)
 
@@ -216,13 +235,17 @@ class ScratchPad(group._Group):
     """
 
     def __init__(
-        self, name="scratchpad", dropdowns: List[config.DropDown] = None, label="", single=False
+        self,
+        name="scratchpad",
+        dropdowns: list[config.DropDown] | None = None,
+        label="",
+        single=False,
     ):
         group._Group.__init__(self, name, label=label)
         self._dropdownconfig = {dd.name: dd for dd in dropdowns} if dropdowns is not None else {}
-        self.dropdowns: Dict[str, DropDownToggler] = {}
-        self._spawned: Dict[str, Match] = {}
-        self._to_hide: List[str] = []
+        self.dropdowns: dict[str, DropDownToggler] = {}
+        self._spawned: dict[str, _Match] = {}
+        self._to_hide: list[str] = []
         self._single = single
 
     def _check_unsubscribe(self):
@@ -242,8 +265,7 @@ class ScratchPad(group._Group):
         if name not in self._spawned:
             if not self._spawned:
                 hook.subscribe.client_new(self.on_client_new)
-
-            pid = self.qtile.cmd_spawn(ddconfig.command)
+            pid = self.qtile.spawn(ddconfig.command)
             self._spawned[name] = ddconfig.match or Match(net_wm_pid=pid)
 
     def on_client_new(self, client, *args, **kwargs):
@@ -282,7 +304,6 @@ class ScratchPad(group._Group):
         name = None
         for name, dd in self.dropdowns.items():
             if dd.window is client:
-                dd.unsubscribe()
                 del self.dropdowns[name]
                 break
         self._check_unsubscribe()
@@ -291,7 +312,7 @@ class ScratchPad(group._Group):
         """
         hook method which is called if window float state is changed.
         If the current associated window is not floated (any more) the window
-        and process is detached from DRopDown, thus the next call to Show
+        and process is detached from DropDown, thus the next call to Show
         will spawn a new process.
         """
         name = None
@@ -303,7 +324,8 @@ class ScratchPad(group._Group):
                     break
         self._check_unsubscribe()
 
-    def cmd_dropdown_toggle(self, name):
+    @expose_command()
+    def dropdown_toggle(self, name):
         """
         Toggle visibility of named DropDown.
         """
@@ -317,14 +339,16 @@ class ScratchPad(group._Group):
             if name in self._dropdownconfig:
                 self._spawn(self._dropdownconfig[name])
 
-    def cmd_hide_all(self):
+    @expose_command()
+    def hide_all(self):
         """
         Hide all scratchpads.
         """
         for d in self.dropdowns.values():
             d.hide()
 
-    def cmd_dropdown_reconfigure(self, name, **kwargs):
+    @expose_command()
+    def dropdown_reconfigure(self, name, **kwargs):
         """
         reconfigure the named DropDown configuration.
         Note that changed attributes only have an effect on spawning the window.
@@ -336,7 +360,8 @@ class ScratchPad(group._Group):
             if hasattr(dd, attr):
                 setattr(dd, attr, value)
 
-    def cmd_dropdown_info(self, name=None):
+    @expose_command()
+    def dropdown_info(self, name=None):
         """
         Get information on configured or currently active DropDowns.
         If name is None, a list of all dropdown names is returned.
@@ -348,7 +373,7 @@ class ScratchPad(group._Group):
         elif name in self._dropdownconfig:
             return self._dropdownconfig[name].info()
         else:
-            raise ValueError('No DropDown named "%s".' % name)
+            raise ValueError(f'No DropDown named "{name}".')
 
     def get_state(self):
         """
@@ -361,7 +386,7 @@ class ScratchPad(group._Group):
             state.append((name, client_wid, dd.visible))
         return state
 
-    def restore_state(self, state, restart: bool):
+    def restore_state(self, state, restart: bool) -> list[int]:
         """
         Restore the state of existing dropdown windows. Used for restoring state across
         Qtile restarts (`restart` == True) or config reloads (`restart` == False).
