@@ -25,8 +25,8 @@ import enum
 import os
 import shlex
 from functools import partial
-from typing import Dict
 
+from libqtile.command.base import expose_command
 from libqtile.log_utils import logger
 from libqtile.widget import base
 
@@ -43,16 +43,13 @@ class Backlight(base.InLoopPollText):
     """A simple widget to show the current brightness of a monitor.
 
     If the change_command parameter is set to None, the widget will attempt to
-    use the interface at /sys/class to change brightness. Depending on the
-    setup, the user may need to be added to the video group to have permission
-    to write to this interface. This depends on having the correct udev rules
-    the brightness file; these are typically installed alongside brightness
-    tools such as brightnessctl (which changes the group to 'video') so
-    installing that is an easy way to get it working.
+    use the interface at /sys/class to change brightness. This depends on
+    having the correct udev rules, so be sure Qtile's udev rules are installed
+    correctly.
 
     You can also bind keyboard shortcuts to the backlight widget with:
 
-    .. code-block: python
+    .. code-block:: python
 
         from libqtile.widget import backlight
         Key(
@@ -67,7 +64,7 @@ class Backlight(base.InLoopPollText):
         )
     """
 
-    filenames = {}  # type: Dict
+    filenames: dict = {}
 
     defaults = [
         ("backlight_name", "acpi_video0", "ACPI name of a backlight device"),
@@ -85,6 +82,7 @@ class Backlight(base.InLoopPollText):
         ("step", 10, "Percent of backlight every scroll changed"),
         ("format", "{percent:2.0%}", "Display format"),
         ("change_command", "xbacklight -set {0}", "Execute command to change value"),
+        ("min_brightness", 0, "Minimum brightness percentage"),
     ]
 
     def __init__(self, **config):
@@ -105,8 +103,8 @@ class Backlight(base.InLoopPollText):
 
         self.add_callbacks(
             {
-                "Button4": partial(self.cmd_change_backlight, ChangeDirection.UP),
-                "Button5": partial(self.cmd_change_backlight, ChangeDirection.DOWN),
+                "Button4": partial(self.change_backlight, ChangeDirection.UP),
+                "Button5": partial(self.change_backlight, ChangeDirection.DOWN),
             }
         )
 
@@ -117,11 +115,11 @@ class Backlight(base.InLoopPollText):
 
     def _load_file(self, path):
         try:
-            with open(path, "r") as f:
+            with open(path) as f:
                 return float(f.read().strip())
         except FileNotFoundError:
-            logger.debug("Failed to get %s" % path)
-            raise RuntimeError("Unable to read status for {}".format(os.path.basename(path)))
+            logger.debug("Failed to get %s", path)
+            raise RuntimeError(f"Unable to read status for {os.path.basename(path)}")
 
     def _get_info(self):
         brightness = self._load_file(self.brightness_file)
@@ -132,7 +130,7 @@ class Backlight(base.InLoopPollText):
         try:
             percent = self._get_info()
         except RuntimeError as e:
-            return "Error: {}".format(e)
+            return f"Error: {e}"
 
         return self.format.format(percent=percent)
 
@@ -144,21 +142,20 @@ class Backlight(base.InLoopPollText):
                     f.write(str(round(value)))
             except PermissionError:
                 logger.warning(
-                    "Cannot set brightness: no write permission for {0}".format(
-                        self.brightness_file
-                    )
+                    "Cannot set brightness: no write permission for %s", self.brightness_file
                 )
         else:
             self.call_process(shlex.split(self.change_command.format(value)))
 
-    def cmd_change_backlight(self, direction, step=None):
+    @expose_command()
+    def change_backlight(self, direction, step=None):
         if not step:
             step = self.step
         if self._future and not self._future.done():
             return
         new = now = self._get_info() * 100
         if direction is ChangeDirection.DOWN:
-            new = max(now - step, 0)
+            new = max(now - step, self.min_brightness)
         elif direction is ChangeDirection.UP:
             new = min(now + step, 100)
         if new != now:
