@@ -1,4 +1,3 @@
-# -*- coding:utf-8 -*-
 # Copyright (c) 2015, Roger Duran. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -18,33 +17,36 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
 import os
-from . import base
 
-class DF(base.ThreadedPollText):
+from libqtile.widget import base
+
+
+class DF(base.ThreadPoolText):
     """Disk Free Widget
 
     By default the widget only displays if the space is less than warn_space.
     """
-    orientations = base.ORIENTATION_HORIZONTAL
+
     defaults = [
-        ('partition', '/', 'the partition to check space'),
-        ('warn_color', 'ff0000', 'Warning color'),
-        ('warn_space', 2, 'Warning space in scale defined by the ``measure`` option.'),
-        ('visible_on_warn', True, 'Only display if warning'),
-        ('measure', "G", "Measurement (G, M, B)"),
-        ('format', '{p} ({uf}{m})',
-            'String format (p: partition, s: size, '
-            'f: free space, uf: user free space, m: measure)'),
-        ('update_interval', 60, 'The update interval.'),
+        ("partition", "/", "the partition to check space"),
+        ("warn_color", "ff0000", "Warning color"),
+        ("warn_space", 2, "Warning space in scale defined by the ``measure`` option."),
+        ("visible_on_warn", True, "Only display if warning"),
+        ("measure", "G", "Measurement (G, M, B)"),
+        (
+            "format",
+            "{p} ({uf}{m}|{r:.0f}%)",
+            "String format (p: partition, s: size, "
+            "f: free space, uf: user free space, m: measure, r: ratio (uf/s))",
+        ),
+        ("update_interval", 60, "The update interval."),
     ]
 
-    measures = {"G": 1024 * 1024 * 1024,
-                "M": 1024 * 1024,
-                "B": 1024}
+    measures = {"G": 1024 * 1024 * 1024, "M": 1024 * 1024, "B": 1024}
+
     def __init__(self, **config):
-        base.ThreadedPollText.__init__(self, **config)
+        base.ThreadPoolText.__init__(self, "", **config)
         self.add_defaults(DF.defaults)
         self.user_free = 0
         self.calc = self.measures[self.measure]
@@ -55,7 +57,7 @@ class DF(base.ThreadedPollText):
         else:
             self.layout.colour = self.foreground
 
-        base.ThreadedPollText.draw(self)
+        base.ThreadPoolText.draw(self)
 
     def poll(self):
         statvfs = os.statvfs(self.partition)
@@ -67,7 +69,13 @@ class DF(base.ThreadedPollText):
         if self.visible_on_warn and self.user_free >= self.warn_space:
             text = ""
         else:
-            text = self.format.format(p=self.partition, s=size, f=free,
-                    uf=self.user_free, m=self.measure)
+            text = self.format.format(
+                p=self.partition,
+                s=size,
+                f=free,
+                uf=self.user_free,
+                m=self.measure,
+                r=(size - self.user_free) / size * 100,
+            )
 
         return text
