@@ -31,21 +31,24 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import cairocffi
-
-from . import base
-from libqtile.log_utils import logger
-from os import statvfs
+import itertools
+import operator
 import time
-import platform
+from os import statvfs
+
+import cairocffi
+import psutil
+
+from libqtile.log_utils import logger
+from libqtile.widget import base
 
 __all__ = [
-    'CPUGraph',
-    'MemoryGraph',
-    'SwapGraph',
-    'NetGraph',
-    'HDDGraph',
-    'HDDBusyGraph',
+    "CPUGraph",
+    "MemoryGraph",
+    "SwapGraph",
+    "NetGraph",
+    "HDDGraph",
+    "HDDBusyGraph",
 ]
 
 
@@ -73,6 +76,11 @@ class _Graph(base._Widget):
         self.oldtime = time.time()
         self.lag_cycles = 0
 
+    def _configure(self, qtile, bar):
+        super()._configure(qtile, bar)
+        if self.type == "box":
+            self.drawer.ctx.set_antialias(cairocffi.ANTIALIAS_NONE)
+
     def timer_setup(self):
         self.timeout_add(self.frequency, self.update)
 
@@ -84,47 +92,59 @@ class _Graph(base._Widget):
     def graphheight(self):
         return self.bar.height - self.margin_y * 2 - self.border_width * 2
 
+    def step(self):
+        return self.graphwidth / float(self.samples)
+
+    def _for_each_step(self, values):
+        yield from enumerate(
+            itertools.islice(
+                values,
+                max(int(-(self.graphwidth / self.step()) + len(values)), 0),
+                len(values),
+            )
+        )
+
+    def _prepare_context(self):
+        self.drawer.ctx.set_line_join(cairocffi.LINE_JOIN_ROUND)
+        if self.graph_color is not None:
+            self.drawer.set_source_rgb(self.graph_color)
+        self.drawer.ctx.set_line_width(self.line_width)
+
     def draw_box(self, x, y, values):
-        step = self.graphwidth / float(self.samples)
-        self.drawer.set_source_rgb(self.graph_color)
-        for val in values:
+        self._prepare_context()
+        for _, val in self._for_each_step(values):
             val = self.val(val)
-            self.drawer.fillrect(x, y - val, step, val)
-            x += step
+            self.drawer.ctx.rectangle(x, y - val, self.step(), val)
+            x += self.step()
+        self.drawer.ctx.fill()
+        self.drawer.ctx.stroke()
 
     def draw_line(self, x, y, values):
-        step = self.graphwidth / float(self.samples - 1)
-        self.drawer.ctx.set_line_join(cairocffi.LINE_JOIN_ROUND)
-        self.drawer.set_source_rgb(self.graph_color)
-        self.drawer.ctx.set_line_width(self.line_width)
-        for val in values:
+        self._prepare_context()
+        for _, val in self._for_each_step(values):
             self.drawer.ctx.line_to(x, y - self.val(val))
-            x += step
+            x += self.step()
         self.drawer.ctx.stroke()
 
     def draw_linefill(self, x, y, values):
-        step = self.graphwidth / float(self.samples - 2)
-        self.drawer.ctx.set_line_join(cairocffi.LINE_JOIN_ROUND)
-        self.drawer.set_source_rgb(self.graph_color)
-        self.drawer.ctx.set_line_width(self.line_width)
-        for index, val in enumerate(values):
-            self.drawer.ctx.line_to(x + index * step, y - self.val(val))
+        self._prepare_context()
+        for index, val in self._for_each_step(values):
+            self.drawer.ctx.line_to(x + index * self.step(), y - self.val(val))
         self.drawer.ctx.stroke_preserve()
         self.drawer.ctx.line_to(
-            x + (len(values) - 1) * step,
-            y - 1 + self.line_width / 2.0
+            x + (len(values) - 1) * self.step(), y - 1 + self.line_width / 2.0
         )
         self.drawer.ctx.line_to(x, y - 1 + self.line_width / 2.0)
         self.drawer.set_source_rgb(self.fill_color)
         self.drawer.ctx.fill()
 
     def val(self, val):
-        if self.start_pos == 'bottom':
+        if self.start_pos == "bottom":
             return val
-        elif self.start_pos == 'top':
+        elif self.start_pos == "top":
             return -val
         else:
-            raise ValueError("Unknown starting position: %s." % self.start_pos)
+            raise ValueError(f"Unknown starting position: {self.start_pos}.")
 
     def draw(self):
         self.drawer.clear(self.background or self.bar.background)
@@ -140,10 +160,10 @@ class _Graph(base._Widget):
             self.drawer.ctx.stroke()
         x = self.margin_x + self.border_width
         y = self.margin_y + self.border_width
-        if self.start_pos == 'bottom':
+        if self.start_pos == "bottom":
             y += self.graphheight
-        elif not self.start_pos == 'top':
-            raise ValueError("Unknown starting position: %s." % self.start_pos)
+        elif not self.start_pos == "top":
+            raise ValueError(f"Unknown starting position: {self.start_pos}.")
         k = 1.0 / (self.maxvalue or 1)
         scaled = [self.graphheight * val * k for val in reversed(self.values)]
 
@@ -154,9 +174,9 @@ class _Graph(base._Widget):
         elif self.type == "linefill":
             self.draw_linefill(x, y, scaled)
         else:
-            raise ValueError("Unknown graph type: %s." % self.type)
+            raise ValueError(f"Unknown graph type: {self.type}.")
 
-        self.drawer.draw(offsetx=self.offset, width=self.width)
+        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
 
     def push(self, value):
         if self.lag_cycles > self.samples:
@@ -165,7 +185,7 @@ class _Graph(base._Widget):
             self.lag_cycles = 1
 
         self.values = ([value] * min(self.samples, self.lag_cycles)) + self.values
-        self.values = self.values[:self.samples]
+        self.values = self.values[: self.samples]
 
         if not self.fixed_upper_bound:
             self.maxvalue = max(self.values)
@@ -185,7 +205,13 @@ class _Graph(base._Widget):
 
 
 class CPUGraph(_Graph):
-    """Display CPU usage graph"""
+    """Display CPU usage graph.
+
+    Widget requirements: psutil_.
+
+    .. _psutil: https://pypi.org/project/psutil/
+    """
+
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
         ("core", "all", "Which core to show (all/0/1/2/...)"),
@@ -200,30 +226,19 @@ class CPUGraph(_Graph):
         self.oldvalues = self._getvalues()
 
     def _getvalues(self):
-        proc = '/proc/stat'
-        if platform.system() == "FreeBSD":
-            proc = '/compat/linux' + proc
+        if isinstance(self.core, int):
+            if self.core > psutil.cpu_count() - 1:
+                raise ValueError(f"No such core: {self.core}")
+            cpu = psutil.cpu_times(percpu=True)[self.core]
+        else:
+            cpu = psutil.cpu_times()
 
-        with open(proc) as file:
-            lines = file.readlines()
+        user = cpu.user * 100
+        nice = cpu.nice * 100
+        sys = cpu.system * 100
+        idle = cpu.idle * 100
 
-            # default to all cores (first line)
-            line = lines.pop(0)
-
-            # core specified, grab the corresponding line
-            if isinstance(self.core, int):
-                # we already removed the first line from the list,
-                # so it's 0 indexed now :D
-                line = lines[self.core]
-
-                if not line.startswith("cpu%s" % self.core):
-                    raise ValueError("No such core: %s" % self.core)
-            if platform.system() == 'FreeBSD':
-                name, user, nice, sys, idle = line.split(None, 4)
-            else:
-                name, user, nice, sys, idle, iowait, tail = line.split(None, 6)
-
-            return (int(user), int(nice), int(sys), int(idle))
+        return (int(user), int(nice), int(sys), int(idle))
 
     def update_graph(self):
         nval = self._getvalues()
@@ -242,82 +257,86 @@ class CPUGraph(_Graph):
         self.oldvalues = nval
 
 
-def get_meminfo():
-    val = {}
-    proc = '/proc/meminfo'
-    if platform.system() == "FreeBSD":
-        proc = "/compat/linux" + proc
-    with open(proc) as file:
-        for line in file:
-            if line.lstrip().startswith("total"):
-                pass
-            else:
-                key, tail = line.strip().split(':')
-                uv = tail.split()
-                val[key] = int(uv[0])
-    val['MemUsed'] = val['MemTotal'] - val['MemFree']
-    return val
-
-
 class MemoryGraph(_Graph):
-    """Displays a memory usage graph"""
+    """Displays a memory usage graph.
+
+    Widget requirements: psutil_.
+
+    .. _psutil: https://pypi.org/project/psutil/
+    """
+
     orientations = base.ORIENTATION_HORIZONTAL
     fixed_upper_bound = True
 
     def __init__(self, **config):
         _Graph.__init__(self, **config)
         val = self._getvalues()
-        self.maxvalue = val['MemTotal']
+        self.maxvalue = val["MemTotal"]
 
-        mem = val['MemTotal'] - val['MemFree'] - val['Buffers'] - val['Cached']
+        mem = val["MemTotal"] - val["MemFree"] - val["Buffers"] - val["Cached"]
         self.fulfill(mem)
 
     def _getvalues(self):
-        return get_meminfo()
+        val = {}
+        mem = psutil.virtual_memory()
+        val["MemTotal"] = int(mem.total / 1024 / 1024)
+        val["MemFree"] = int(mem.free / 1024 / 1024)
+        val["Buffers"] = int(mem.buffers / 1024 / 1024)
+        val["Cached"] = int(mem.cached / 1024 / 1024)
+        return val
 
     def update_graph(self):
         val = self._getvalues()
-        self.push(
-            val['MemTotal'] - val['MemFree'] - val['Buffers'] - val['Cached']
-        )
+        self.push(val["MemTotal"] - val["MemFree"] - val["Buffers"] - val["Cached"])
 
 
 class SwapGraph(_Graph):
-    """Display a swap info graph"""
+    """Display a swap info graph.
+
+    Widget requirements: psutil_.
+
+    .. _psutil: https://pypi.org/project/psutil/
+    """
+
     orientations = base.ORIENTATION_HORIZONTAL
     fixed_upper_bound = True
 
     def __init__(self, **config):
         _Graph.__init__(self, **config)
         val = self._getvalues()
-        self.maxvalue = val['SwapTotal']
-        swap = val['SwapTotal'] - val['SwapFree'] - val.get('SwapCached', 0)
+        self.maxvalue = val["SwapTotal"]
+        swap = val["SwapTotal"] - val["SwapFree"]
         self.fulfill(swap)
 
     def _getvalues(self):
-        return get_meminfo()
+        val = {}
+        swap = psutil.swap_memory()
+        val["SwapTotal"] = int(swap.total / 1024 / 1024)
+        val["SwapFree"] = int(swap.free / 1024 / 1024)
+        return val
 
     def update_graph(self):
         val = self._getvalues()
 
-        swap = val['SwapTotal'] - val['SwapFree'] - val.get('SwapCached', 0)
+        swap = val["SwapTotal"] - val["SwapFree"]
 
         # can change, swapon/off
-        if self.maxvalue != val['SwapTotal']:
-            self.maxvalue = val['SwapTotal']
+        if self.maxvalue != val["SwapTotal"]:
+            self.maxvalue = val["SwapTotal"]
             self.fulfill(swap)
         self.push(swap)
 
 
 class NetGraph(_Graph):
-    """Display a network usage graph"""
+    """Display a network usage graph.
+
+    Widget requirements: psutil_.
+
+    .. _psutil: https://pypi.org/project/psutil/"""
+
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
-        (
-            "interface",
-            "auto",
-            "Interface to display info for ('auto' for detection)"
-        ),
+        ("interface", "auto", "Interface to display info for ('auto' for detection)"),
         ("bandwidth_type", "down", "down(load)/up(load)"),
     ]
 
@@ -329,73 +348,67 @@ class NetGraph(_Graph):
                 self.interface = self.get_main_iface()
             except RuntimeError:
                 logger.warning(
-                    "NetGraph - Automatic interface detection failed, "
-                    "falling back to 'eth0'"
+                    "NetGraph - Automatic interface detection failed, falling back to 'eth0'"
                 )
                 self.interface = "eth0"
-        self.filename = '/sys/class/net/{interface}/statistics/{type}'.format(
-            interface=self.interface,
-            type=self.bandwidth_type == 'down' and 'rx_bytes' or 'tx_bytes'
-        )
+        if self.bandwidth_type != "down" and self.bandwidth_type != "up":
+            raise ValueError(f"bandwidth type {self.bandwidth_type} not known!")
         self.bytes = 0
-        self.bytes = self._getValues()
+        self.bytes = self._get_values()
 
-    def _getValues(self):
-        try:
-            with open(self.filename) as file:
-                val = int(file.read())
-                rval = val - self.bytes
-                self.bytes = val
-                return rval
-        except IOError:
-            return 0
+    def _get_values(self):
+        net = psutil.net_io_counters(pernic=True)
+        if self.bandwidth_type == "up":
+            return net[self.interface].bytes_sent
+        if self.bandwidth_type == "down":
+            return net[self.interface].bytes_recv
 
     def update_graph(self):
-        val = self._getValues()
-        self.push(val)
+        val = self._get_values()
+        change = val - self.bytes
+        self.bytes = val
+        self.push(change)
 
     @staticmethod
     def get_main_iface():
-        def make_route(line):
-            return dict(zip(['iface', 'dest'], line.split()))
-        with open('/proc/net/route', 'r') as fp:
-            lines = fp.readlines()
-        routes = [make_route(line) for line in lines[1:]]
-        try:
-            return next(
-                (r for r in routes if not int(r['dest'], 16)),
-                routes[0]
-            )['iface']
-        except (KeyError, IndexError, ValueError):
-            raise RuntimeError('No valid interfaces available')
+        # XXX: psutil doesn't have the facility to get the main interface,
+        # so I'll just return the interface that has received the most traffic.
+        #
+        # I could do this with netifaces, but that's another dependency.
+        #
+        # Oh. and there is probably a better way to do this.
+
+        net = psutil.net_io_counters(pernic=True)
+        iface = {}
+        for entry in net:
+            iface[entry] = net[entry].bytes_recv
+        return sorted(iface.items(), key=operator.itemgetter(1))[-1][0]
 
 
 class HDDGraph(_Graph):
     """Display HDD free or used space graph"""
+
     fixed_upper_bound = True
     orientations = base.ORIENTATION_HORIZONTAL
-    defaults = [
-        ("path", "/", "Partition mount point."),
-        ("space_type", "used", "free/used")
-    ]
+    defaults = [("path", "/", "Partition mount point."), ("space_type", "used", "free/used")]
 
     def __init__(self, **config):
         _Graph.__init__(self, **config)
         self.add_defaults(HDDGraph.defaults)
         stats = statvfs(self.path)
         self.maxvalue = stats.f_blocks * stats.f_frsize
-        values = self._getValues()
+        values = self._get_values()
         self.fulfill(values)
 
-    def _getValues(self):
+    def _get_values(self):
         stats = statvfs(self.path)
-        if self.space_type == 'used':
+        if self.space_type == "used":
             return (stats.f_blocks - stats.f_bfree) * stats.f_frsize
         else:
             return stats.f_bavail * stats.f_frsize
 
     def update_graph(self):
-        val = self._getValues()
+        val = self._get_values()
         self.push(val)
 
 
@@ -406,29 +419,27 @@ class HDDBusyGraph(_Graph):
     based on ``io_ticks``'s value.  See
     https://www.kernel.org/doc/Documentation/block/stat.txt
     """
+
     orientations = base.ORIENTATION_HORIZONTAL
-    defaults = [
-        ("device", "sda", "Block device to display info for")
-    ]
+    defaults = [("device", "sda", "Block device to display info for")]
 
     def __init__(self, **config):
         _Graph.__init__(self, **config)
         self.add_defaults(HDDBusyGraph.defaults)
-        self.path = '/sys/block/{dev}/stat'.format(
-            dev=self.device
-        )
+        self.path = f"/sys/block/{self.device}/stat"
         self._prev = 0
 
-    def _getActivity(self):
+    def _get_values(self):
         try:
             # io_ticks is field number 9
             with open(self.path) as f:
                 io_ticks = int(f.read().split()[9])
-        except IOError:
+        except OSError:
             return 0
         activity = io_ticks - self._prev
         self._prev = io_ticks
         return activity
 
     def update_graph(self):
-        self.push(self._getActivity())
+        val = self._get_values()
+        self.push(val)
