@@ -32,27 +32,28 @@
 
 import itertools
 from functools import partial
-from typing import Any, List, Tuple
+from typing import Any
 
-from libqtile import bar, hook
+from libqtile import hook
 from libqtile.widget import base
 
 
 class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
-    defaults = [
+    defaults: list[tuple[str, Any, str]] = [
         ("borderwidth", 3, "Current group border width"),
         ("center_aligned", True, "center-aligned group box"),
-    ]  # type: List[Tuple[str, Any, str]]
+        ("markup", False, "Whether or not to use pango markup"),
+    ]
 
     def __init__(self, **config):
-        base._TextBox.__init__(self, width=bar.CALCULATED, **config)
+        base._TextBox.__init__(self, **config)
         self.add_defaults(_GroupBase.defaults)
         self.add_defaults(base.PaddingMixin.defaults)
         self.add_defaults(base.MarginMixin.defaults)
 
     def box_width(self, groups):
         width, _ = self.drawer.max_layout_size(
-            [i.label for i in groups], self.font, self.fontsize
+            [self.fmt.format(i.label) for i in groups], self.font, self.fontsize, self.markup
         )
         return width + self.padding_x * 2 + self.borderwidth * 2
 
@@ -64,21 +65,30 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
             self.fontsize = max(calc, 1)
 
         self.layout = self.drawer.textlayout(
-            "", "ffffff", self.font, self.fontsize, self.fontshadow
+            "", "ffffff", self.font, self.fontsize, self.fontshadow, markup=self.markup
         )
         self.setup_hooks()
 
-    def setup_hooks(self):
-        def hook_response(*args, **kwargs):
-            self.bar.draw()
+    def _hook_response(self, *args, **kwargs):
+        self.bar.draw()
 
-        hook.subscribe.client_managed(hook_response)
-        hook.subscribe.client_urgent_hint_changed(hook_response)
-        hook.subscribe.client_killed(hook_response)
-        hook.subscribe.setgroup(hook_response)
-        hook.subscribe.group_window_add(hook_response)
-        hook.subscribe.current_screen_change(hook_response)
-        hook.subscribe.changegroup(hook_response)
+    def setup_hooks(self):
+        hook.subscribe.client_managed(self._hook_response)
+        hook.subscribe.client_urgent_hint_changed(self._hook_response)
+        hook.subscribe.client_killed(self._hook_response)
+        hook.subscribe.setgroup(self._hook_response)
+        hook.subscribe.group_window_add(self._hook_response)
+        hook.subscribe.current_screen_change(self._hook_response)
+        hook.subscribe.changegroup(self._hook_response)
+
+    def remove_hooks(self):
+        hook.unsubscribe.client_managed(self._hook_response)
+        hook.unsubscribe.client_urgent_hint_changed(self._hook_response)
+        hook.unsubscribe.client_killed(self._hook_response)
+        hook.unsubscribe.setgroup(self._hook_response)
+        hook.unsubscribe.group_window_add(self._hook_response)
+        hook.unsubscribe.current_screen_change(self._hook_response)
+        hook.unsubscribe.changegroup(self._hook_response)
 
     def drawbox(
         self,
@@ -93,7 +103,7 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
         line=False,
         highlighted=False,
     ):
-        self.layout.text = text
+        self.layout.text = self.fmt.format(text)
         self.layout.font_family = self.font
         self.layout.font_size = self.fontsize
         self.layout.colour = textcolor
@@ -131,6 +141,10 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
         else:
             framed.draw(offset, y, rounded)
 
+    def finalize(self):
+        self.remove_hooks()
+        base._TextBox.finalize(self)
+
 
 class AGroupBox(_GroupBase):
     """A widget that graphically displays the current group"""
@@ -144,7 +158,7 @@ class AGroupBox(_GroupBase):
 
     def _configure(self, qtile, bar):
         _GroupBase._configure(self, qtile, bar)
-        self.add_callbacks({"Button1": partial(self.bar.screen.cmd_next_group, warp=False)})
+        self.add_callbacks({"Button1": partial(self.bar.screen.next_group, warp=False)})
 
     def calculate_length(self):
         return self.box_width(self.qtile.groups) + self.margin_x * 2
@@ -224,13 +238,12 @@ class GroupBox(_GroupBase):
             "Hide groups that have no windows and that are not displayed on any screen.",
         ),
         ("spacing", None, "Spacing between groups" "(if set to None, will be equal to margin_x)"),
+        ("toggle", True, "Enable toggling of group when clicking on same group name"),
     ]
 
     def __init__(self, **config):
         _GroupBase.__init__(self, **config)
         self.add_defaults(GroupBox.defaults)
-        if self.spacing is None:
-            self.spacing = self.margin_x
         self.clicked = None
         self.click = None
 
@@ -244,6 +257,11 @@ class GroupBox(_GroupBase):
             )
         self.add_callbacks(default_callbacks)
 
+    def _configure(self, qtile, bar):
+        _GroupBase._configure(self, qtile, bar)
+        if self.spacing is None:
+            self.spacing = self.margin_x
+
     @property
     def groups(self):
         """
@@ -252,20 +270,15 @@ class GroupBox(_GroupBase):
         their label. Groups with an empty string as label are never contained.
         Groups that are not named in visible_groups are not returned.
         """
+        groups = filter(lambda g: g.label, self.qtile.groups)
+
         if self.hide_unused:
-            if self.visible_groups:
-                return [
-                    g
-                    for g in self.qtile.groups
-                    if g.label and (g.windows or g.screen) and g.name in self.visible_groups
-                ]
-            else:
-                return [g for g in self.qtile.groups if g.label and (g.windows or g.screen)]
-        else:
-            if self.visible_groups:
-                return [g for g in self.qtile.groups if g.label and g.name in self.visible_groups]
-            else:
-                return [g for g in self.qtile.groups if g.label]
+            groups = filter(lambda g: g.windows or g.screen, groups)
+
+        if self.visible_groups:
+            groups = filter(lambda g: g.name in self.visible_groups, groups)
+
+        return list(groups)
 
     def get_clicked_group(self):
         group = None
@@ -312,7 +325,7 @@ class GroupBox(_GroupBase):
 
     def go_to_group(self, group):
         if group:
-            if self.bar.screen.group != group or not self.disable_drag:
+            if self.bar.screen.group != group or not self.disable_drag or not self.toggle:
                 self.bar.screen.set_group(group, warp=False)
             else:
                 self.bar.screen.toggle_group(group, warp=False)
@@ -322,7 +335,7 @@ class GroupBox(_GroupBase):
         if button not in (5, 4):
             group = self.get_clicked_group()
             if group and self.clicked:
-                group.cmd_switch_groups(self.clicked.name)
+                group.switch_groups(self.clicked.name)
                 self.clicked = None
 
     def calculate_length(self):
@@ -332,7 +345,7 @@ class GroupBox(_GroupBase):
         return width
 
     def group_has_urgent(self, group):
-        return len([w for w in group.windows if w.urgent]) > 0
+        return any(w.urgent for w in group.windows)
 
     def draw(self):
         self.drawer.clear(self.background or self.bar.background)
