@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2011 Timo Schmiade
 # Copyright (c) 2012 Phil Jackson
 # Copyright (c) 2013 Tao Sauvage
@@ -24,9 +23,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
 import mailbox
 import os.path
-from typing import Dict
 
 from libqtile.widget import base
 
@@ -101,14 +101,14 @@ class Maildir(base.ThreadPoolText):
         if color is None:  # default to self.foreground
             return s
 
-        return s.join(('<span foreground="{}">'.format(color), "</span>"))
+        return s.join((f'<span foreground="{color}">', "</span>"))
 
-    def format_text(self, state: Dict[str, int]) -> str:
+    def format_text(self, state: dict[str, int]) -> str:
         """Converts the state of the subfolders to a string
 
         Parameters
         ==========
-        state: Dict[str, int]
+        state: dict[str, int]
             a dictionary mapping subfolder labels to new mail values
 
         Returns
