@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2010-2011 Aldo Cortesi
 # Copyright (c) 2010 Philip Kranz
 # Copyright (c) 2011 Mounier Florian
@@ -31,16 +30,17 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
 import abc
 import glob
 import os
 import pickle
 import string
 from collections import deque
-from typing import List, Optional, Tuple
 
-from libqtile import bar, hook, pangocffi, utils
-from libqtile.command.base import CommandObject, SelectError
+from libqtile import hook, pangocffi, utils
+from libqtile.command.base import CommandObject, SelectError, expose_command
 from libqtile.command.client import InteractiveCommandClient
 from libqtile.command.interface import CommandError, QtileCommandInterface
 from libqtile.log_utils import logger
@@ -53,7 +53,7 @@ class AbstractCompleter(metaclass=abc.ABCMeta):
         pass
 
     @abc.abstractmethod
-    def actual(self) -> Optional[str]:
+    def actual(self) -> str | None:
         pass
 
     @abc.abstractmethod
@@ -61,9 +61,14 @@ class AbstractCompleter(metaclass=abc.ABCMeta):
         pass
 
     @abc.abstractmethod
-    def complete(self, txt: str) -> str:
-        """Perform the requested completion on the given text"""
-        pass  # pragma: no cover
+    def complete(self, txt: str, aliases: dict[str, str] | None = None) -> str:
+        """
+        Perform the requested completion on the given text.
+
+        The completer can optionally support aliases, which map strings to commands. The
+        completer should include the aliases in the completion results.
+        """
+        # pragma: no cover
 
 
 class NullCompleter(AbstractCompleter):
@@ -76,7 +81,7 @@ class NullCompleter(AbstractCompleter):
     def reset(self) -> None:
         pass
 
-    def complete(self, txt: str) -> str:
+    def complete(self, txt: str, _aliases: dict[str, str] | None = None) -> str:
         return txt
 
 
@@ -84,17 +89,17 @@ class FileCompleter(AbstractCompleter):
     def __init__(self, qtile, _testing=False) -> None:
         self._testing = _testing
         self.qtile = qtile
-        self.thisfinal = None  # type: Optional[str]
-        self.lookup = None  # type: Optional[List[Tuple[str, str]]]
+        self.thisfinal = None  # type: str | None
+        self.lookup = None  # type: list[tuple[str, str]] | None
         self.reset()
 
-    def actual(self) -> Optional[str]:
+    def actual(self) -> str | None:
         return self.thisfinal
 
     def reset(self) -> None:
         self.lookup = None
 
-    def complete(self, txt: str) -> str:
+    def complete(self, txt: str, _aliases: dict[str, str] | None = None) -> str:
         """Returns the next completion for txt, or None if there is no completion"""
         if self.lookup is None:
             self.lookup = []
@@ -128,18 +133,18 @@ class QshCompleter(AbstractCompleter):
     def __init__(self, qtile: CommandObject) -> None:
         q = QtileCommandInterface(qtile)
         self.client = InteractiveCommandClient(q)
-        self.thisfinal = None  # type: Optional[str]
+        self.thisfinal = None  # type: str | None
         self.reset()
 
-    def actual(self) -> Optional[str]:
+    def actual(self) -> str | None:
         return self.thisfinal
 
     def reset(self) -> None:
-        self.lookup = None  # type: Optional[List[Tuple[str, str]]]
+        self.lookup = None  # type: list[tuple[str, str]] | None
         self.path = ""
         self.offset = -1
 
-    def complete(self, txt: str) -> str:
+    def complete(self, txt: str, _aliases: dict[str, str] | None = None) -> str:
         txt = txt.lower()
         if self.lookup is None:
             self.lookup = []
@@ -149,7 +154,7 @@ class QshCompleter(AbstractCompleter):
             if len(self.path) > 0:
                 self.path += "."
 
-            contains_cmd = "self.client.%s_contains" % self.path
+            contains_cmd = f"self.client.{self.path}_contains"
             try:
                 contains = eval(contains_cmd)
             except AttributeError:
@@ -158,7 +163,7 @@ class QshCompleter(AbstractCompleter):
                 if obj.lower().startswith(term):
                     self.lookup.append((obj, obj))
 
-            commands_cmd = "self.client.%scommands()" % self.path
+            commands_cmd = f"self.client.{self.path}commands()"
             try:
                 commands = eval(commands_cmd)
             except (CommandError, AttributeError):
@@ -181,11 +186,11 @@ class QshCompleter(AbstractCompleter):
 class GroupCompleter(AbstractCompleter):
     def __init__(self, qtile: CommandObject) -> None:
         self.qtile = qtile
-        self.thisfinal = None  # type: Optional[str]
-        self.lookup = None  # type: Optional[List[Tuple[str, str]]]
+        self.thisfinal = None  # type: str | None
+        self.lookup = None  # type: list[tuple[str, str]] | None
         self.offset = -1
 
-    def actual(self) -> Optional[str]:
+    def actual(self) -> str | None:
         """Returns the current actual value"""
         return self.thisfinal
 
@@ -193,12 +198,12 @@ class GroupCompleter(AbstractCompleter):
         self.lookup = None
         self.offset = -1
 
-    def complete(self, txt: str) -> str:
+    def complete(self, txt: str, _aliases: dict[str, str] | None = None) -> str:
         """Returns the next completion for txt, or None if there is no completion"""
         txt = txt.lower()
         if not self.lookup:
             self.lookup = []
-            for group in self.qtile.groups_map.keys():  # type: ignore
+            for group in self.qtile.groups_map.keys():
                 if group.lower().startswith(txt):
                     self.lookup.append((group, group))
 
@@ -217,11 +222,11 @@ class GroupCompleter(AbstractCompleter):
 class WindowCompleter(AbstractCompleter):
     def __init__(self, qtile: CommandObject) -> None:
         self.qtile = qtile
-        self.thisfinal = None  # type: Optional[str]
-        self.lookup = None  # type: Optional[List[Tuple[str, str]]]
+        self.thisfinal = None  # type: str | None
+        self.lookup = None  # type: list[tuple[str, str]] | None
         self.offset = -1
 
-    def actual(self) -> Optional[str]:
+    def actual(self) -> str | None:
         """Returns the current actual value"""
         return self.thisfinal
 
@@ -229,11 +234,11 @@ class WindowCompleter(AbstractCompleter):
         self.lookup = None
         self.offset = -1
 
-    def complete(self, txt: str) -> str:
+    def complete(self, txt: str, _aliases: dict[str, str] | None = None) -> str:
         """Returns the next completion for txt, or None if there is no completion"""
         if self.lookup is None:
             self.lookup = []
-            for wid, window in self.qtile.windows_map.items():  # type: ignore
+            for wid, window in self.qtile.windows_map.items():
                 if window.group and window.name.lower().startswith(txt):
                     self.lookup.append((window.name, wid))
 
@@ -260,12 +265,12 @@ class CommandCompleter:
     DEFAULTPATH = "/bin:/usr/bin:/usr/local/bin"
 
     def __init__(self, qtile, _testing=False):
-        self.lookup = None  # type: Optional[List[Tuple[str, str]]]
+        self.lookup = None  # type: list[tuple[str, str]] | None
         self.offset = -1
-        self.thisfinal = None  # type: Optional[str]
+        self.thisfinal = None  # type: str | None
         self._testing = _testing
 
-    def actual(self) -> Optional[str]:
+    def actual(self) -> str | None:
         """Returns the current actual value"""
         return self.thisfinal
 
@@ -276,7 +281,7 @@ class CommandCompleter:
         self.lookup = None
         self.offset = -1
 
-    def complete(self, txt: str) -> str:
+    def complete(self, txt: str, aliases: dict[str, str] | None = None) -> str:
         """Returns the next completion for txt, or None if there is no completion"""
         if self.lookup is None:
             # Lookup is a set of (display value, actual value) tuples.
@@ -301,13 +306,19 @@ class CommandCompleter:
                 for d in dirs:
                     try:
                         d = os.path.expanduser(d)
-                        for cmd in glob.iglob(os.path.join(d, "%s*" % txt)):
+                        for cmd in glob.iglob(os.path.join(d, f"{txt}*")):
                             if self.executable(cmd):
                                 self.lookup.append(
                                     (os.path.basename(cmd), cmd),
                                 )
                     except OSError:
                         pass
+
+            if aliases:
+                for alias in aliases:
+                    if alias.startswith(txt):
+                        self.lookup.append((alias, aliases[alias]))
+
             self.lookup.sort()
             self.offset = -1
             self.lookup.append((txt, txt))
@@ -352,12 +363,12 @@ class Prompt(base._TextBox):
         ("visual_bell_time", 0.2, "Visual bell duration (in seconds)."),
     ]
 
-    def __init__(self, name="prompt", **config) -> None:
-        base._TextBox.__init__(self, "", bar.CALCULATED, **config)
+    def __init__(self, **config) -> None:
+        base._TextBox.__init__(self, "", **config)
         self.add_defaults(Prompt.defaults)
-        self.name = name
         self.active = False
-        self.completer = None  # type: Optional[AbstractCompleter]
+        self.completer = None  # type: AbstractCompleter | None
+        self.aliases: dict[str, str] | None = None
 
         # If history record is on, get saved history or create history record
         if self.record_history:
@@ -397,6 +408,11 @@ class Prompt(base._TextBox):
             if self.active and not win == self.bar.window:
                 self._unfocus()
 
+        def prevent_focus_steal(win):
+            if self.active:
+                win.can_steal_focus = False
+
+        hook.subscribe.client_new(prevent_focus_steal)
         hook.subscribe.client_focus(f)
 
         # Define key handlers (action to do when a specific key is hit)
@@ -430,7 +446,13 @@ class Prompt(base._TextBox):
             self.original_background = self.background
 
     def start_input(
-        self, prompt, callback, complete=None, strict_completer=False, allow_empty_input=False
+        self,
+        prompt,
+        callback,
+        complete=None,
+        strict_completer=False,
+        allow_empty_input=False,
+        aliases: dict[str, str] | None = None,
     ) -> None:
         """Run the prompt
 
@@ -457,6 +479,9 @@ class Prompt(base._TextBox):
             available.
         allow_empty_input :
             When True, an empty value will still call the callback function
+        aliases :
+            Dictionary mapping aliases to commands. If the entered command is a key in
+            this dict, the command it maps to will be executed instead.
         """
 
         if self.cursor and self.cursorblink and not self.active:
@@ -469,6 +494,7 @@ class Prompt(base._TextBox):
         self.show_cursor = self.cursor
         self.cursor_position = 0
         self.callback = callback
+        self.aliases = aliases
         self.completer = self.completers[complete](self.qtile)
         self.strict_completer = strict_completer
         self.allow_empty_input = allow_empty_input
@@ -493,9 +519,9 @@ class Prompt(base._TextBox):
 
     def _highlight_text(self, text) -> str:
         color = utils.hex(self.cursor_color)
-        text = '<span foreground="{0}">{1}</span>'.format(color, text)
+        text = f'<span foreground="{color}">{text}</span>'
         if self.show_cursor:
-            text = "<u>{}</u>".format(text)
+            text = f"<u>{text}</u>"
         return text
 
     def _update(self) -> None:
@@ -509,7 +535,7 @@ class Prompt(base._TextBox):
                 for text in (txt1, txt2, txt3):
                     text = pangocffi.markup_escape_text(text)
                 txt2 = self._highlight_text(txt2)
-                self.text = "{0}{1}{2}{3}".format(txt1, txt2, txt3, cursor)
+                self.text = f"{txt1}{txt2}{txt3}{cursor}"
             else:
                 self.text = pangocffi.markup_escape_text(self.text)
                 self.text += self._highlight_text(cursor)
@@ -521,7 +547,7 @@ class Prompt(base._TextBox):
     def _trigger_complete(self) -> None:
         # Trigger the auto completion in user input
         assert self.completer is not None
-        self.user_input = self.completer.complete(self.user_input)
+        self.user_input = self.completer.complete(self.user_input, self.aliases)
         self.cursor_position = len(self.user_input)
 
     def _history_to_input(self) -> None:
@@ -674,10 +700,12 @@ class Prompt(base._TextBox):
             del self.key
         self._update()
 
-    def cmd_fake_keypress(self, key: str) -> None:
+    @expose_command()
+    def fake_keypress(self, key: str) -> None:
         self.process_key_press(self.qtile.core.keysym_from_name(key))
 
-    def cmd_info(self):
+    @expose_command()
+    def info(self):
         """Returns a dictionary of info for this object"""
         return dict(
             name=self.name,
@@ -686,7 +714,8 @@ class Prompt(base._TextBox):
             active=self.active,
         )
 
-    def cmd_exec_general(self, prompt, object_name, cmd_name, selector=None, completer=None):
+    @expose_command()
+    def exec_general(self, prompt, object_name, cmd_name, selector=None, completer=None):
         """
         Execute a cmd of any object. For example layout, group, window, widget
         , etc with a string that is obtained from start_input.
@@ -736,10 +765,10 @@ class Prompt(base._TextBox):
         self.history = {x: self._dedup_deque(self.history[x]) for x in self.completers}
 
     def _dedup_deque(self, dq):
-        return deque(_LastUpdatedOrderedDict.fromkeys(dq))
+        return deque(_LastUpdatedOrdereddict.fromkeys(dq))
 
 
-class _LastUpdatedOrderedDict(dict):
+class _LastUpdatedOrdereddict(dict):
     """Store items in the order the keys were last added."""
 
     def __setitem__(self, key, value):
