@@ -45,6 +45,7 @@ class Icon(window._Window):
 
     def __init__(self, win, qtile, systray):
         window._Window.__init__(self, win, qtile)
+        self.hidden = True
         self.systray = systray
         # win.get_name() may return None when apps provide a temporary window before the icon window
         # we need something in self.name in order to sort icons so we use the window's WID.
@@ -95,7 +96,9 @@ class Icon(window._Window):
     handle_UnmapNotify = handle_DestroyNotify  # noqa: N815
 
 
-class Systray(window._Window, base._Widget):
+# Mypy doesn't like the inheritance of height and width as _Widget's
+# properties are read only but _Window's have a getter and setter.
+class Systray(base._Widget, window._Window):  # type: ignore[misc]
     """
     A widget that manages system tray.
 
@@ -118,6 +121,8 @@ class Systray(window._Window, base._Widget):
 
     orientations = base.ORIENTATION_BOTH
 
+    supported_backends = {"x11"}
+
     defaults = [
         ("icon_size", 20, "Icon width"),
         ("padding", 5, "Padding between icons"),
@@ -178,11 +183,11 @@ class Systray(window._Window, base._Widget):
         )
 
         conn.conn.core.SetSelectionOwner(
-            win.wid, atoms["_NET_SYSTEM_TRAY_S{:d}".format(self.screen)], xcffib.CurrentTime
+            win.wid, atoms[f"_NET_SYSTEM_TRAY_S{self.screen:d}"], xcffib.CurrentTime
         )
         data = [
             xcffib.CurrentTime,
-            atoms["_NET_SYSTEM_TRAY_S{:d}".format(self.screen)],
+            atoms[f"_NET_SYSTEM_TRAY_S{self.screen:d}"],
             win.wid,
             0,
             0,
@@ -273,21 +278,25 @@ class Systray(window._Window, base._Widget):
     def finalize(self):
         base._Widget.finalize(self)
         atoms = self.conn.atoms
-        self.conn.conn.core.SetSelectionOwner(
-            0,
-            atoms["_NET_SYSTEM_TRAY_S{:d}".format(self.screen)],
-            xcffib.CurrentTime,
-        )
-        self.hide()
 
-        root = self.qtile.core._root.wid
-        for icon in self.tray_icons:
-            self.conn.conn.core.ReparentWindow(icon.window.wid, root, 0, 0)
-        self.conn.conn.flush()
+        try:
+            self.conn.conn.core.SetSelectionOwner(
+                0,
+                atoms[f"_NET_SYSTEM_TRAY_S{self.screen:d}"],
+                xcffib.CurrentTime,
+            )
+            self.hide()
+
+            root = self.qtile.core._root.wid
+            for icon in self.tray_icons:
+                self.conn.conn.core.ReparentWindow(icon.window.wid, root, 0, 0)
+            self.conn.conn.flush()
+
+            self.conn.conn.core.DestroyWindow(self.wid)
+        except xcffib.ConnectionException:
+            self.hidden = True  # Usually set in self.hide()
 
         del self.qtile.windows_map[self.wid]
-        self.conn.conn.core.DestroyWindow(self.wid)
-
         Systray._instances -= 1
 
     def info(self):
