@@ -34,13 +34,15 @@ import re
 import subprocess
 
 from libqtile import bar
+from libqtile.command.base import expose_command
+from libqtile.log_utils import logger
 from libqtile.widget import base
 
 __all__ = [
     "Volume",
 ]
 
-re_vol = re.compile(r"\[(\d?\d?\d?)%\]")
+re_vol = re.compile(r"(\d?\d?\d?)%")
 
 
 class Volume(base._TextBox):
@@ -67,11 +69,33 @@ class Volume(base._TextBox):
             "Use emoji to display volume states, only if ``theme_path`` is not set."
             "The specified font needs to contain the correct unicode characters.",
         ),
+        (
+            "emoji_list",
+            ["\U0001f507", "\U0001f508", "\U0001f509", "\U0001f50a"],
+            "List of emojis/font-symbols to display volume states, only if ``emoji`` is set."
+            " List contains 4 symbols, from lowest volume to highest.",
+        ),
         ("mute_command", None, "Mute command"),
+        ("mute_foreground", None, "Foreground color for mute volume."),
+        ("mute_format", "M", "Format to display when volume is muted."),
+        ("unmute_format", "{volume}%", "Format of text to display when volume is not muted."),
         ("volume_app", None, "App to control volume"),
         ("volume_up_command", None, "Volume up command"),
         ("volume_down_command", None, "Volume down command"),
-        ("get_volume_command", None, "Command to get the current volume"),
+        (
+            "get_volume_command",
+            None,
+            "Command to get the current volume. "
+            "The expected output should include 1-3 numbers and a ``%`` sign.",
+        ),
+        ("check_mute_command", None, "Command to check mute status"),
+        (
+            "check_mute_string",
+            "[off]",
+            "String expected from check_mute_command when volume is muted."
+            "When the output of the command matches this string, the"
+            "audio source is treated as muted.",
+        ),
         (
             "step",
             2,
@@ -81,17 +105,18 @@ class Volume(base._TextBox):
     ]
 
     def __init__(self, **config):
-        base._TextBox.__init__(self, "0", width=bar.CALCULATED, **config)
+        base._TextBox.__init__(self, "0", **config)
         self.add_defaults(Volume.defaults)
         self.surfaces = {}
         self.volume = None
+        self.is_mute = False
 
         self.add_callbacks(
             {
-                "Button1": self.cmd_mute,
-                "Button3": self.cmd_run_app,
-                "Button4": self.cmd_increase_vol,
-                "Button5": self.cmd_decrease_vol,
+                "Button1": self.mute,
+                "Button3": self.run_app,
+                "Button4": self.increase_vol,
+                "Button5": self.decrease_vol,
             }
         )
 
@@ -100,6 +125,7 @@ class Volume(base._TextBox):
             self.length_type = bar.STATIC
             self.length = 0
         base._TextBox._configure(self, qtile, parent_bar)
+        self.unmute_foreground = self.foreground
 
     def timer_setup(self):
         self.timeout_add(self.update_interval, self.update)
@@ -116,16 +142,17 @@ class Volume(base._TextBox):
             cmd.extend(["-D", str(self.device)])
 
         cmd.extend([x for x in args])
-        return cmd
+        return subprocess.list2cmdline(cmd)
 
     def button_press(self, x, y, button):
         base._TextBox.button_press(self, x, y, button)
         self.draw()
 
     def update(self):
-        vol = self.get_volume()
-        if vol != self.volume:
+        vol, muted = self.get_volume()
+        if vol != self.volume or muted != self.is_mute:
             self.volume = vol
+            self.is_mute = muted
             # Update the underlying canvas size before actually attempting
             # to figure out how big it is and draw it.
             self._update_drawer()
@@ -133,9 +160,12 @@ class Volume(base._TextBox):
         self.timeout_add(self.update_interval, self.update)
 
     def _update_drawer(self):
+        if self.mute_foreground is not None:
+            self.foreground = self.mute_foreground if self.is_mute else self.unmute_foreground
+
         if self.theme_path:
             self.drawer.clear(self.background or self.bar.background)
-            if self.volume <= 0:
+            if self.volume <= 0 or self.is_mute:
                 img_name = "audio-volume-muted"
             elif self.volume <= 30:
                 img_name = "audio-volume-low"
@@ -147,19 +177,24 @@ class Volume(base._TextBox):
             self.drawer.ctx.set_source(self.surfaces[img_name])
             self.drawer.ctx.paint()
         elif self.emoji:
-            if self.volume <= 0:
-                self.text = u"\U0001f507"
+            if len(self.emoji_list) < 4:
+                self.emoji_list = ["\U0001f507", "\U0001f508", "\U0001f509", "\U0001f50a"]
+                logger.warning(
+                    "Emoji list given has less than 4 items. Falling back to default emojis."
+                )
+
+            if self.volume <= 0 or self.is_mute:
+                self.text = self.emoji_list[0]
             elif self.volume <= 30:
-                self.text = u"\U0001f508"
+                self.text = self.emoji_list[1]
             elif self.volume < 80:
-                self.text = u"\U0001f509"
+                self.text = self.emoji_list[2]
             elif self.volume >= 80:
-                self.text = u"\U0001f50a"
+                self.text = self.emoji_list[3]
         else:
-            if self.volume == -1:
-                self.text = "M"
-            else:
-                self.text = "{}%".format(self.volume)
+            self.text = (
+                self.mute_format if self.is_mute or self.volume < 0 else self.unmute_format
+            ).format(volume=self.volume)
 
     def setup_images(self):
         from libqtile import images
@@ -180,24 +215,27 @@ class Volume(base._TextBox):
 
     def get_volume(self):
         try:
-            get_volume_cmd = self.create_amixer_command("sget", self.channel)
-
-            if self.get_volume_command:
+            if self.get_volume_command is not None:
                 get_volume_cmd = self.get_volume_command
+            else:
+                get_volume_cmd = self.create_amixer_command("sget", self.channel)
 
-            mixer_out = self.call_process(get_volume_cmd)
+            mixer_out = subprocess.getoutput(get_volume_cmd)
         except subprocess.CalledProcessError:
-            return -1
+            return -1, False
 
-        if "[off]" in mixer_out:
-            return -1
+        check_mute = mixer_out
+        if self.check_mute_command:
+            check_mute = subprocess.getoutput(self.check_mute_command)
+
+        muted = self.check_mute_string in check_mute
 
         volgroups = re_vol.search(mixer_out)
         if volgroups:
-            return int(volgroups.groups()[0])
+            return int(volgroups.groups()[0]), muted
         else:
             # this shouldn't happen
-            return -1
+            return -1, muted
 
     def draw(self):
         if self.theme_path:
@@ -205,28 +243,38 @@ class Volume(base._TextBox):
         else:
             base._TextBox.draw(self)
 
-    def cmd_increase_vol(self):
+    @expose_command()
+    def increase_vol(self):
         if self.volume_up_command is not None:
-            subprocess.call(self.volume_up_command, shell=True)
+            volume_up_cmd = self.volume_up_command
         else:
-            subprocess.call(
-                self.create_amixer_command("-q", "sset", self.channel, "{}%+".format(self.step))
+            volume_up_cmd = self.create_amixer_command(
+                "-q", "sset", self.channel, f"{self.step}%+"
             )
 
-    def cmd_decrease_vol(self):
+        subprocess.call(volume_up_cmd, shell=True)
+
+    @expose_command()
+    def decrease_vol(self):
         if self.volume_down_command is not None:
-            subprocess.call(self.volume_down_command, shell=True)
+            volume_down_cmd = self.volume_down_command
         else:
-            subprocess.call(
-                self.create_amixer_command("-q", "sset", self.channel, "{}%-".format(self.step))
+            volume_down_cmd = self.create_amixer_command(
+                "-q", "sset", self.channel, f"{self.step}%-"
             )
 
-    def cmd_mute(self):
-        if self.mute_command is not None:
-            subprocess.call(self.mute_command, shell=True)
-        else:
-            subprocess.call(self.create_amixer_command("-q", "sset", self.channel, "toggle"))
+        subprocess.call(volume_down_cmd, shell=True)
 
-    def cmd_run_app(self):
+    @expose_command()
+    def mute(self):
+        if self.mute_command is not None:
+            mute_cmd = self.mute_command
+        else:
+            mute_cmd = self.create_amixer_command("-q", "sset", self.channel, "toggle")
+
+        subprocess.call(mute_cmd, shell=True)
+
+    @expose_command()
+    def run_app(self):
         if self.volume_app is not None:
             subprocess.Popen(self.volume_app, shell=True)
