@@ -20,12 +20,10 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-import logging
-import multiprocessing
 
 import pytest
 
-from libqtile.backend import base
+from libqtile.backend.base import drawer
 from test.helpers import BareConfig, TestManager
 
 
@@ -105,30 +103,8 @@ def backend(request, backend_name, xephyr, wayland_session):
 
 
 @pytest.fixture(scope="function")
-def log_queue():
-    """Creates a new Queue for logging messages run in a backend process."""
-    yield multiprocessing.Queue(-1)
-
-
-@pytest.fixture(scope="function")
-def logger(caplog, log_queue):
-    """
-    Connects logging messages in the backend to a logger in the main thread
-    and returns a caplog fixture  which can access those messages.
-    """
-    root = logging.getLogger()
-    listener = logging.handlers.QueueListener(log_queue, *root.handlers)
-    listener.start()
-
-    yield caplog
-
-    listener.stop()
-
-
-@pytest.fixture(scope="function")
-def manager_nospawn(request, backend, log_queue):
+def manager_nospawn(request, backend):
     with TestManager(backend, request.config.getoption("--debuglog")) as manager:
-        manager.log_queue = log_queue
         yield manager
 
 
@@ -140,6 +116,14 @@ def manager(request, manager_nospawn):
     yield manager_nospawn
 
 
+@pytest.fixture(scope="function")
+def manager_withlogs(request, manager_nospawn):
+    config = getattr(request, "param", BareConfig)
+
+    manager_nospawn.start(config, want_logs=True)
+    yield manager_nospawn
+
+
 @pytest.fixture(scope="function")
 def fake_window():
     """
@@ -153,6 +137,6 @@ def fake_window():
         window = _NestedWindow()
 
         def create_drawer(self, width, height):
-            return base.Drawer(None, self, width, height)
+            return drawer.Drawer(None, self, width, height)
 
     return FakeWindow()
