@@ -50,12 +50,12 @@ def test_crashme_init(manager_nospawn, minimal_conf_noscreen):
 
     # Left click generates ZeroDivisionError
     with pytest.raises(CommandException) as e_info:
-        topbar.fake_button_press(0, "top", 0, 0, button=1)
+        topbar.fake_button_press(0, 0, button=1)
 
     assert e_info.match("ZeroDivisionError")
 
     # Simulate right click to trigger parse_markup error
     with pytest.raises(CommandException) as e_info:
-        topbar.fake_button_press(0, "top", 0, 0, button=3)
+        topbar.fake_button_press(0, 0, button=3)
 
     assert e_info.match("parse_markup[(][)] failed")
