@@ -25,7 +25,6 @@ from test.widgets.conftest import FakeBar
 
 
 def test_widgetbox_widget(fake_qtile, fake_window):
-
     tb_one = TextBox(name="tb_one", text="TB ONE")
     tb_two = TextBox(name="tb_two", text="TB TWO")
 
@@ -45,7 +44,7 @@ def test_widgetbox_widget(fake_qtile, fake_window):
     assert fakebar.widgets == [widget_box]
 
     # Open box
-    widget_box.cmd_toggle()
+    widget_box.toggle()
 
     # Check it's open
     assert widget_box.box_is_open
@@ -54,7 +53,7 @@ def test_widgetbox_widget(fake_qtile, fake_window):
     assert fakebar.widgets == [widget_box, tb_one, tb_two]
 
     # Close box
-    widget_box.cmd_toggle()
+    widget_box.toggle()
 
     # Check it's closed
     assert not widget_box.box_is_open
@@ -66,12 +65,25 @@ def test_widgetbox_widget(fake_qtile, fake_window):
     widget_box.close_button_location = "right"
 
     # Re-open box with new layout
-    widget_box.cmd_toggle()
+    widget_box.toggle()
 
     # Now widgetbox is on the right
     assert fakebar.widgets == [tb_one, tb_two, widget_box]
 
 
+def test_widgetbox_start_opened(manager_nospawn, minimal_conf_noscreen):
+    config = minimal_conf_noscreen
+    tbox = TextBox(text="Text Box")
+    widget_box = WidgetBox(widgets=[tbox], start_opened=True)
+    config.screens = [libqtile.config.Screen(top=libqtile.bar.Bar([widget_box], 10))]
+
+    manager_nospawn.start(config)
+
+    topbar = manager_nospawn.c.bar["top"]
+    widgets = [w["name"] for w in topbar.info()["widgets"]]
+    assert widgets == ["widgetbox", "textbox"]
+
+
 def test_widgetbox_mirror(manager_nospawn, minimal_conf_noscreen):
     config = minimal_conf_noscreen
     tbox = TextBox(text="Text Box")
@@ -99,10 +111,10 @@ def test_widgetbox_mouse_click(manager_nospawn, minimal_conf_noscreen):
     topbar = manager_nospawn.c.bar["top"]
     assert len(topbar.info()["widgets"]) == 1
 
-    topbar.fake_button_press(0, "top", 0, 0, button=1)
+    topbar.fake_button_press(0, 0, button=1)
     assert len(topbar.info()["widgets"]) == 2
 
-    topbar.fake_button_press(0, "top", 0, 0, button=1)
+    topbar.fake_button_press(0, 0, button=1)
     assert len(topbar.info()["widgets"]) == 1
 
 
@@ -164,3 +176,38 @@ def test_deprecated_configuration(caplog):
     box = WidgetBox([tray])
     assert box.widgets == [tray]
     assert "The use of a positional argument in WidgetBox is deprecated." in caplog.text
+
+
+def test_widgetbox_open_close_commands(manager_nospawn, minimal_conf_noscreen):
+    config = minimal_conf_noscreen
+    tbox = TextBox(text="Text Box")
+    widget_box = WidgetBox(widgets=[tbox])
+    config.screens = [libqtile.config.Screen(top=libqtile.bar.Bar([widget_box], 10))]
+
+    manager_nospawn.start(config)
+
+    topbar = manager_nospawn.c.bar["top"]
+    widget = manager_nospawn.c.widget["widgetbox"]
+
+    def count():
+        return len(topbar.info()["widgets"])
+
+    assert count() == 1
+
+    widget.open()
+    assert count() == 2
+
+    widget.open()
+    assert count() == 2
+
+    widget.close()
+    assert count() == 1
+
+    widget.close()
+    assert count() == 1
+
+    widget.toggle()
+    assert count() == 2
+
+    widget.toggle()
+    assert count() == 1
