@@ -20,6 +20,7 @@
 
 # Widget specific tests
 
+import builtins
 import sys
 from importlib import reload
 from types import ModuleType
@@ -34,6 +35,23 @@ def no_op(*args, **kwargs):
     pass
 
 
+def mock_open(output):
+    class MockOpen:
+        def __init__(self, *args):
+            self.output = output
+
+        def __enter__(self):
+            return self
+
+        def __exit__(*exc):
+            return None
+
+        def read(self):
+            return self.output
+
+    return MockOpen
+
+
 class MockIwlib(ModuleType):
     DATA = {
         "wlan0": {
@@ -44,7 +62,10 @@ class MockIwlib(ModuleType):
             "ESSID": b"QtileNet",
             "Mode": b"Managed",
             "stats": {"quality": 49, "level": 190, "noise": 0, "updated": 75},
-        }
+        },
+        "wlan1": {
+            "ESSID": None,
+        },
     }
 
     @classmethod
@@ -80,3 +101,32 @@ def test_wlan_display(minimal_conf_noscreen, manager_nospawn, patched_wlan, kwar
 
     text = manager_nospawn.c.bar["top"].info()["widgets"][0]["text"]
     assert text == expected
+
+
+def test_wlan_display_escape_essid(
+    minimal_conf_noscreen, manager_nospawn, patched_wlan, monkeypatch
+):
+    """Test escaping of pango markup in ESSID"""
+    monkeypatch.setitem(MockIwlib.DATA["wlan0"], "ESSID", b"A&B")
+    widget = patched_wlan.Wlan(format="{essid}")
+    assert widget.poll() == "A&amp;B"
+
+
+@pytest.mark.parametrize(
+    "kwargs,state,expected",
+    [
+        ({"interface": "wlan1", "use_ethernet": True}, "up", "eth"),
+        ({"interface": "wlan1", "use_ethernet": True}, "down", "Disconnected"),
+        (
+            {"interface": "wlan1", "use_ethernet": True, "ethernet_message": "Wired"},
+            "up",
+            "Wired",
+        ),
+    ],
+)
+def test_ethernet(
+    minimal_conf_noscreen, manager_nospawn, patched_wlan, kwargs, state, expected, monkeypatch
+):
+    monkeypatch.setattr(builtins, "open", mock_open(state))
+    widget = patched_wlan.Wlan(**kwargs)
+    assert widget.poll() == expected
